"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetTemplateEditorAdvancedDeployOverride = exports.FormatTemplateEditorAdvancedDeployHydratedMetadataYAML = exports.GetAlterTableAdvancedHydratedMetadata = exports.GetDropTableAdvancedHydratedMetadata = exports.GetAddTableAdvancedHydratedMetadata = exports.GetStepDeployStrategyFromStepTypesParsed = exports.GetStepDeployStrategyFromStepTypesYAML = exports.EDeployStrategy = void 0;
/* eslint-disable multiline-comment-style */
const Runner = __importStar(require("./Runner"));
const MetadataHydration = __importStar(require("./MetadataHydration"));
const StepTypes = __importStar(require("./StepTypes"));
const Common = __importStar(require("./Common"));
const YAML = __importStar(require("yaml"));
const Logging = __importStar(require("./Logging"));
////////////////////////////////////////////////////////
////////////// Advanced Deploy Notes ///////////////////
////////////////////////////////////////////////////////
/**
 * Advanced Deploy is enabled inside of the node definition (template editor, stepType definitions) w/property "deployStrategy: advanced"
 * Advanced Deploy grants access to both the "current" ("before") and "desired" ("after") state of the node's hydrated metadata which is either
 *      1 - gathered from firestore and the git commit respectively (during deploy), or
 *      2 - gathered from currentState and desiredState inside of templateEditor.advancedDeploy (during template editor render)
 *      3 - provided directly as desiredState by helper functions when using create/validate create, while currentState === null
 *
 * These states of hydrated metadata may be accessed inside of the "create" script:
 *      current state is accessed under: { "currentState": MetadataHydration.HydratedMetadata | null}
 *      desired state is accessed under: { "desiredState": MetadataHydration.HydratedMetadata | null}
 *
 * current and desired state mappings may also be accessed, also standard 'mapping' property has been preserved (see next note re: nodeMetadata shape):
 *      { "currentMapping": Runner.IPhysicalLocations }
 *      { "desiredMapping": Runner.IPhysicalLocations }
 *
 * These all live under nodeMetadata:
 *      nodeMetadata: {
 *          currentMapping: {...}
 *          desiredMapping: {...}
 *          currentState: {...}
 *          desiredState: {...}
 *          mapping: {...} // same as desiredState mapping
 *          destinationName: string
 *          operationName: string
 *      }
 *
 * In the case that one or the other doesn't exist, it will be null at run time
 *      i.e. with ADD TABLE, currentState === null
 *           with DELETE TABLE, desiredState === null
 */
////////////////////////////////////////////////////////
//////////////////////////////////////////////////////// 
////////////////////////////////////////////////////////
const logger = Logging.GetLogger(Logging.LoggingArea.AdvancedDeploy);
var EDeployStrategy;
(function (EDeployStrategy) {
    EDeployStrategy["default"] = "default";
    EDeployStrategy["advanced"] = "advanced";
})(EDeployStrategy = exports.EDeployStrategy || (exports.EDeployStrategy = {}));
const GetNodeMetadataForAdvancedDeployStrategyInternal = (hydratedMetadataCurrent, hydratedMetadataDesired, nodeMetadata, currentMapping, desiredMapping) => {
    return {
        currentState: hydratedMetadataCurrent,
        desiredState: hydratedMetadataDesired,
        operationName: nodeMetadata.name,
        destinationName: nodeMetadata.locationName,
        mapping: (desiredMapping || currentMapping),
        currentMapping: !!currentMapping ? currentMapping : undefined,
        desiredMapping: !!desiredMapping ? desiredMapping : undefined,
    };
};
const GetStepDeployStrategyFromStepTypesYAML = (stepCounter, steps, stepTypes) => {
    const countersByDeployStrategy = {};
    Object.keys(EDeployStrategy).forEach(key => countersByDeployStrategy[key] = []);
    const sqlType = steps[stepCounter].operation.sqlType;
    const stepTypesParsed = StepTypes.ParseStepTypesYAML(stepTypes);
    const metadata = StepTypes.getStepTypeMetadataSafe(stepTypesParsed, sqlType);
    logger.info(`Getting deploy strategy for step ${stepCounter} with sql type ${sqlType}`);
    const deployStrategy = Common.getValueSafe(metadata, ["nodeMetadataSpec", "deployStrategy"], EDeployStrategy.default);
    if (!metadata.nodeMetadataSpec || !metadata.nodeMetadataSpec.deployStrategy) { // this is valid case
        logger.info(`Using default deploy strategy for step ${stepCounter} since nodeMetadataSpec or deployStrategy was undefined (valid)`);
    }
    else {
        logger.info(`Using ${deployStrategy} deploy strategy for step ${stepCounter}`);
    }
    return deployStrategy;
};
exports.GetStepDeployStrategyFromStepTypesYAML = GetStepDeployStrategyFromStepTypesYAML;
const GetStepDeployStrategyFromStepTypesParsed = (stepCounter, steps, stepTypesParsed) => {
    const countersByDeployStrategy = {};
    Object.keys(EDeployStrategy).forEach(key => countersByDeployStrategy[key] = []);
    const sqlType = steps[stepCounter].operation.sqlType;
    if (!stepTypesParsed[sqlType]) {
        Common.assert(Logging.LoggingArea.AdvancedDeploy, false, `sqlType ${sqlType} does not exist!`);
    }
    const nodeMetadataSpec = stepTypesParsed[sqlType].metadata.nodeMetadataSpec;
    logger.info(`Getting deploy strategy for step ${stepCounter} with sql type ${sqlType}`);
    if (sqlType === Runner.ESQLType.source) {
        logger.info(`Using default deploy strategy for Source step`);
        return EDeployStrategy.default;
    }
    else if (!nodeMetadataSpec) {
        logger.info(`Using default deploy strategy for step ${stepCounter} since nodeMetadataSpec null (valid)`);
        return EDeployStrategy.default;
    }
    let deployStrategy;
    if (nodeMetadataSpec.deployStrategy) {
        deployStrategy = nodeMetadataSpec.deployStrategy;
    }
    else {
        const nodeMetadataSpecParsed = Runner.parseYAMLString(nodeMetadataSpec);
        deployStrategy = Common.getValueSafe(nodeMetadataSpecParsed, ["deployStrategy"], EDeployStrategy.default);
    }
    Common.assert(Logging.LoggingArea.AdvancedDeploy, deployStrategy in EDeployStrategy, `Invalid deploy strategy ${deployStrategy}`);
    logger.info(`Using ${deployStrategy} deploy strategy for step ${stepCounter}`);
    return deployStrategy;
};
exports.GetStepDeployStrategyFromStepTypesParsed = GetStepDeployStrategyFromStepTypesParsed;
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////// Helpers used during deploy to create hydratedmaetadata for workspace edits  ///////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
const GetAddTableAdvancedHydratedMetadata = (nodeMetadata, stepID, specifiedIndex, mapping, allSteps, stepTypesYaml, workspaces, runTimeParameters) => {
    logger.info(`Advanced Deploy: Getting hydrated metadata for ADD TABLE for step ${stepID}`);
    const hydratedMetadataDesired = MetadataHydration.hydrateMetadata(nodeMetadata, stepID, allSteps, stepTypesYaml, workspaces, specifiedIndex, runTimeParameters);
    return GetNodeMetadataForAdvancedDeployStrategyInternal(undefined, hydratedMetadataDesired, nodeMetadata, null, mapping);
};
exports.GetAddTableAdvancedHydratedMetadata = GetAddTableAdvancedHydratedMetadata;
const GetDropTableAdvancedHydratedMetadata = (nodeMetadata, stepID, specifiedIndex, mapping, workspaceData, workspaces, stepTypesYaml, runTimeParameters) => {
    logger.info(`Advanced Deploy: Getting hydrated metadata for DROP TABLE for step ${stepID}`);
    const hydratedMetadataDesired = MetadataHydration.hydrateMetadata(nodeMetadata, stepID, workspaceData.steps, stepTypesYaml, workspaces, specifiedIndex, runTimeParameters);
    return GetNodeMetadataForAdvancedDeployStrategyInternal(hydratedMetadataDesired, undefined, nodeMetadata, mapping, null);
};
exports.GetDropTableAdvancedHydratedMetadata = GetDropTableAdvancedHydratedMetadata;
const GetAlterTableAdvancedHydratedMetadata = (oldNodeOperation, newNodeOperation, oldWorkspaceData, newWorkspaceData, stepID, specifiedIndex, oldMappings, newMappings, oldWorkspaces, newWorkspaces, runTimeParameters) => {
    logger.info(`Advanced Deploy: Getting hydrated metadata for ALTER TABLE for step ${stepID}`);
    const newAndOldStepTypes = Object.assign(Object.assign({}, oldWorkspaceData.stepTypes), newWorkspaceData.stepTypes);
    const hydratedMetadataCurrent = MetadataHydration.hydrateMetadata(oldNodeOperation, stepID, oldWorkspaceData.steps, newAndOldStepTypes, oldWorkspaces, specifiedIndex, runTimeParameters);
    const hydratedMetadataDesired = MetadataHydration.hydrateMetadata(newNodeOperation, stepID, newWorkspaceData.steps, newAndOldStepTypes, newWorkspaces, specifiedIndex, runTimeParameters);
    return GetNodeMetadataForAdvancedDeployStrategyInternal(hydratedMetadataCurrent, hydratedMetadataDesired, newNodeOperation, oldMappings, newMappings);
};
exports.GetAlterTableAdvancedHydratedMetadata = GetAlterTableAdvancedHydratedMetadata;
const FormatTemplateEditorAdvancedDeployHydratedMetadataYAML = (hydratedMetadata) => {
    return {
        currentStateYAML: YAML.stringify({ currentState: hydratedMetadata }),
        desiredStateYAML: YAML.stringify({ desiredState: hydratedMetadata })
    };
};
exports.FormatTemplateEditorAdvancedDeployHydratedMetadataYAML = FormatTemplateEditorAdvancedDeployHydratedMetadataYAML;
const GetTemplateEditorAdvancedDeployOverride = (editorAdvancedDeployMetadata, nodeMetadata, physicalLocations) => {
    const { currentState, desiredState } = editorAdvancedDeployMetadata;
    logger.info(`Metadata Override: Getting hydrated metadata for template editor metadata override`);
    /**
     * current/desired states may be null - unfortunately null does not play well with the template renderer,
     * so have to use undefined to keep what's in place with the python api
     * */
    const hydratedCurrentState = currentState ? Runner.parseYAMLString(currentState) : undefined;
    const hydratedDesiredState = desiredState ? Runner.parseYAMLString(desiredState) : undefined;
    return GetNodeMetadataForAdvancedDeployStrategyInternal(hydratedCurrentState === null || hydratedCurrentState === void 0 ? void 0 : hydratedCurrentState.currentState, hydratedDesiredState === null || hydratedDesiredState === void 0 ? void 0 : hydratedDesiredState.desiredState, nodeMetadata, {}, physicalLocations);
};
exports.GetTemplateEditorAdvancedDeployOverride = GetTemplateEditorAdvancedDeployOverride;
//# sourceMappingURL=AdvancedDeployOperations.js.map