"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateDeployRequestObject = exports.CreatePlan = exports.CreatePlanCLI = void 0;
const Workspaces = __importStar(require("./Workspaces"));
const Runner = __importStar(require("./Runner"));
const Logging = __importStar(require("./Logging"));
const SchedulerOperations = __importStar(require("./SchedulerOperations"));
const MetadataHydration = __importStar(require("./MetadataHydration"));
/**
 * In flux, will change
 */
const DeployLogger = Logging.GetLogger(Logging.LoggingArea.Deploy);
const BurncoCD_2676Logger = Logging.GetLogger(Logging.LoggingArea.CD_2676Logger);
/**
 *
 * @param environmentID target environment to deploy to
 * @param fileSystemSettings File System plus path to files where are located
 * @param token Coalesce firestore token (refresh_token for now, id_token after Michael's changes)
 * @param platformType Snowflake for now
 * @returns DeployPlan to be submitted and run by Coalesce Deploy code
 */
const CreatePlanCLI = (environmentID, fileSystemSettings, token, platformType, runtimeParameters) => {
    let logContext;
    return SchedulerOperations.AuthenticateFirebaseTokenAndRetrieveTeamInfoForCLI(token, undefined)
        .then((authInfo) => {
        const { teamInfo, firebase } = authInfo;
        logContext = { orgID: teamInfo.fbTeamID, userID: teamInfo.fbUserID, envID: environmentID };
        const firestore = firebase.firestore();
        return (0, exports.CreatePlan)(environmentID, fileSystemSettings, firestore, teamInfo, platformType, null, runtimeParameters);
    })
        .catch(error => {
        DeployLogger.errorContext(logContext, "CLI Deploy Plan creation error: ", error);
        throw error;
    });
};
exports.CreatePlanCLI = CreatePlanCLI;
//dont calculate plan edits if workspace(target) is in use (deploying) todo (add as step before getWorkspaceEdits)
const CreatePlan = (environmentID, fileSystemSettings, firestore, teamInfo, platformType, commitOID, runTimeParameters) => {
    return new Promise((resolve, reject) => {
        const logContext = { orgID: teamInfo.fbTeamID, userID: teamInfo.fbUserID, envID: environmentID };
        const plan = CreateBaseDeployPlan(1, environmentID);
        const gitDataPromise = () => Workspaces.GetAllWorkspaceDataFromGitCommit(fileSystemSettings, commitOID, environmentID);
        const firestoreDataPromise = () => Workspaces.getAllWorkspaceDataFromFirebase(firestore, teamInfo.fbTeamID, environmentID);
        BurncoCD_2676Logger.infoContext(logContext, `CreateDeployPlan executes at: ${new Date()}`);
        BurncoCD_2676Logger.infoContext(logContext, `Initiating Promise.all in CreateDeployPlan at: ${new Date()}`);
        if (!environmentID) {
            reject(new Error("no environmentID specified"));
            return;
        }
        Promise.all([gitDataPromise(), firestoreDataPromise()])
            .then(results => {
            var _a;
            BurncoCD_2676Logger.infoContext(logContext, `Resolved Promise.all in CreateDeployPlan at: ${new Date()}`);
            const [allGitData, allFirestoreData] = results;
            //don't calculate edits if the user hasn't commited the environment they're planning for
            const hasCurrentWorkspace = !!((_a = allGitData.workspaces) === null || _a === void 0 ? void 0 : _a.workspaces[environmentID]);
            if (!hasCurrentWorkspace) {
                throw new Error(`Workspace with ID: ${environmentID} cannot be found. Have you committed and pushed the data.yml containing this workspace?`);
            }
            const currentLocations = allGitData.workspaces.workspaces[environmentID].mappings;
            allGitData.locations = currentLocations;
            BurncoCD_2676Logger.infoContext(logContext, `Getting workspaceEdits with the following info:
                allGitData: ${allGitData},
                allFirestoreData: ${allFirestoreData},
                platformType: ${platformType}`);
            //order of data in return array from Promise.all is guaranteed to match the order in the array received by Promise.all
            return Workspaces.getWorkspaceEdits(allGitData, allFirestoreData, environmentID, platformType, runTimeParameters);
        })
            .then(editsPlusExtras => {
            const { nodeEdits, AllWorkspaceData } = editsPlusExtras;
            plan.plan = nodeEdits;
            plan.environmentState = AllWorkspaceData;
            plan.runtimeParameters = runTimeParameters;
            BurncoCD_2676Logger.infoContext(logContext, `Resolving the following plan: ${plan}`);
            resolve({ plan, logContext });
        })
            .catch(reject);
    });
};
exports.CreatePlan = CreatePlan;
const CreateBaseDeployPlan = (version, targetEnvironment) => {
    return {
        version,
        targetEnvironment,
        plan: null,
        environmentState: null,
        gitInfo: null,
        runtimeParameters: null,
    };
};
//CreateDeployRequestObject can throw if runTimeParameters fails to parse
const CreateDeployRequestObject = (deployEdits, gitData, teamInfo, commitInfo, environmentID, runTimeParameters) => {
    const gitDataCopy = Object.assign({}, gitData);
    const { oid, commit } = commitInfo;
    //wipe out deployedWorkspaceData, otherwise we nest poorly on the backend
    for (let workspace in gitDataCopy["workspaces"]["workspaces"]) {
        delete gitDataCopy["workspaces"]["workspaces"][workspace]["deployedWorkspaces"];
    }
    let runTimeParametersObject = null;
    if (runTimeParameters) {
        runTimeParametersObject = MetadataHydration.ParseRunTimeParametersStringToObject(runTimeParameters);
    }
    //hook up environmentID for later
    const requestObject = {
        runDetails: { deployCommit: oid, deployCommitMessage: commit.message, fromWorkspaceData: gitDataCopy, toWorkspaceID: environmentID, environmentID: `${environmentID}`, dividedStepDataWithWorkspaceEdits: deployEdits, stepsInRun: Object.keys(deployEdits).length },
        userCredentials: null,
        runType: Runner.ERunType.deploy,
        userID: teamInfo.fbUserID,
        runStatus: Runner.ERunStatus.waitingToRun,
        runHistory: [],
        runTimeParameters: runTimeParametersObject
    };
    return requestObject;
};
exports.CreateDeployRequestObject = CreateDeployRequestObject;
//# sourceMappingURL=DeployOperations.js.map