"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidateEnvironmentMappings = exports.setJob = exports.getNewJob = exports.getJobsStatusCollectionRef = exports.JobStatus = void 0;
const CommonOperations = __importStar(require("./CommonOperations"));
const Firebase = __importStar(require("./Firebase"));
const Logging = __importStar(require("./Logging"));
const Common = __importStar(require("./Common"));
var JobStatus;
(function (JobStatus) {
    JobStatus["waiting"] = "waiting";
    JobStatus["running"] = "running";
    JobStatus["success"] = "success";
    JobStatus["failed"] = "failed";
})(JobStatus = exports.JobStatus || (exports.JobStatus = {}));
//Jobs
const getJobsStatusCollectionRef = (orgConfigDocRef) => {
    return orgConfigDocRef.collection("jobsStatus");
};
exports.getJobsStatusCollectionRef = getJobsStatusCollectionRef;
const getNewJob = (orgConfigDocRef) => {
    const firestore = Firebase.getFirestore();
    return CommonOperations.getNextCounter(firestore, orgConfigDocRef, "jobStatusCounter", 1).then((newJobCounter) => {
        return { jobStatusCounter: newJobCounter, status: JobStatus.running };
    });
};
exports.getNewJob = getNewJob;
const setJob = (orgConfigDocRef, jobInfo) => {
    (0, exports.getJobsStatusCollectionRef)(orgConfigDocRef).doc(jobInfo.jobStatusCounter.toString()).set(jobInfo);
};
exports.setJob = setJob;
/**
 * Function responsible to ensure that an environment's mappings contain no duplicate LocationNames
 * @param environment
 * @returns void
 */
const ValidateEnvironmentMappings = (environment, loggingContext) => {
    const { mappings } = environment;
    if (!mappings)
        return; //valid to have no mappings on a new environment
    const locationNameSet = new Set();
    Object.values(mappings).forEach(mapping => {
        //all locationNames are saved to firestore as uppercase, but don't assume that here
        const { locationName } = mapping;
        Common.assertWithContext(Logging.LoggingArea.StorageLocations, !locationNameSet.has(locationName.toUpperCase()), loggingContext, `${locationName} has duplicate mappings`);
        locationNameSet.add(locationName.toUpperCase());
    });
};
exports.ValidateEnvironmentMappings = ValidateEnvironmentMappings;
//# sourceMappingURL=Environments.js.map