"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllMacros = exports.RefDescriptions = exports.RefMacros = exports.refMacroDefinitions = exports.useMacros = exports.MacroStringSelector = void 0;
const react_1 = require("react");
const react_redux_1 = require("react-redux");
const Common = __importStar(require("./Common"));
const MacroStringSelector = (state) => {
    const userMacros = state.macros;
    return Common.getValueSafe(userMacros, [1, "macroString"], "");
};
exports.MacroStringSelector = MacroStringSelector;
const useMacros = () => {
    const macros = (0, react_redux_1.useSelector)(state => state.macros);
    return (0, react_1.useMemo)(() => {
        return Object.assign(Object.assign({}, macros), exports.refMacroDefinitions);
    }, [macros, exports.refMacroDefinitions]);
};
exports.useMacros = useMacros;
exports.refMacroDefinitions = {
    //returns all references when done, and inserts the strings into the template
    ref: { macroString: `{%- macro ref(location_name, node_name) -%}
        {%- set foundMapping = { 'flag': false } -%}
        {%- for storage in nodeMetadata.mapping -%}
            {%- if nodeMetadata.mapping[storage].locationName ==  location_name -%}
                {%- if foundMapping.update({'flag': true}) -%}{%- endif -%}
                {{-  location_name | export_ref(node_name, ref_type='ref') -}}
                "{{- nodeMetadata.mapping[storage].database -}}"."{{- nodeMetadata.mapping[storage].schema -}}"."{{- node_name -}}"
            {%- endif -%}
        {%- endfor -%}
        {%- if not foundMapping.flag -%}
            {{-  location_name | invalid_export_ref(node_name, ref_type='ref') -}}
        {%- endif -%}
    {%- endmacro -%}` },
    //returns all references when done, doesn't insert the strings into the template
    refLink: { macroString: `{%- macro ref_link(location_name, node_name) -%}
        {%- set foundMapping = { 'flag': false } -%}
        {%- for storage in nodeMetadata.mapping -%}
            {%- if nodeMetadata.mapping[storage].locationName ==  location_name -%}
                {%- if foundMapping.update({'flag': true}) -%}{%- endif -%}
                {{-  location_name | export_ref(node_name, ref_type='ref_link') -}}
            {%- endif -%}
        {%- endfor -%}
        {%- if not foundMapping.flag -%}
            {{-  location_name | invalid_export_ref(node_name, ref_type='ref_link') -}}
        {%- endif -%}
    {%- endmacro -%}` },
    //returns no references when done, and inserts the strings into the template
    noLink: { macroString: `{%- macro ref_no_link(location_name, node_name) -%}
        {%- set foundMapping = { 'flag': false } -%}
        {%- for storage in nodeMetadata.mapping -%}
            {%- if nodeMetadata.mapping[storage].locationName ==  location_name -%}
                {%- if foundMapping.update({'flag': true}) -%}{%- endif -%}
                {{-  location_name | export_ref(node_name, ref_type='ref_no_link') -}}
                "{{- nodeMetadata.mapping[storage].database -}}"."{{- nodeMetadata.mapping[storage].schema -}}"."{{- node_name -}}"
            {%- endif -%}
        {%- endfor -%}
        {%- if not foundMapping.flag -%}
            {{-  location_name | invalid_export_ref(node_name, ref_type='ref_no_link') -}}
        {%- endif -%}
    {%- endmacro -%}` },
    hash: { macroString: `{%- macro hash(columns, algo='MD5', delimiter='||', datatype='CHAR(32)') -%}
    {% for col in columns %}
        {# set algo function #}
        {% set algo_start = algo + '(' %}
        {% set algo_end = ')' %}
        {% if algo | upper == 'SHA256' %}
            {% set algo_start = 'SHA2(' %}
            {% set algo_end = ', 256)' %}
        {% endif %}  
        {# end set algo function #}

        {%- if loop.first %}CAST( {{ algo_start }}{% endif -%}
            NVL(CAST(
                {%  if col is string %} 
                    {{ col }}
                {% elif col.transform | trim != '' %}
                    {{ col.transform }}
                {% elif col.sourceColumns[0] %} 
                    "{{ col.sourceColumns[0].node.name }}"."{{ col.sourceColumns[0].column.name }}"
                {% else %}
                    null
                {%  endif %} 
            AS VARCHAR), 'null')
        {%- if not loop.last %} || {% if delimiter != '' %} '{{ delimiter }}' || {% endif -%} {% endif -%}
        {%- if loop.last %}{{ algo_end }} AS {{ datatype }} ){% endif -%}
    {% endfor %}
{%- endmacro -%}` },
    get_value_by_column_attribute: { macroString: `{%- macro get_value_by_column_attribute(column_attribute, value="name") -%}
    {%- set filtered_columns_by_attribute = columns | selectattr(column_attribute) | list -%}
    {%- if filtered_columns_by_attribute | length > 0 -%}
        {{- (filtered_columns_by_attribute | first)[value] -}}
    {%- else -%}
        ## ERR: COLUMN_NOT_FOUND_WITH_ATTRIBUTE_{{ column_attribute }} ##
    {%- endif -%}
{%- endmacro -%}` },
    newStage: { macroString: `{% macro stage(stage_name, continue_on_failure=true, type="sql") %}
---
coalesce_stage_start
name: {{ stage_name }}
type: {{ type }}
continue_on_failure: {{ continue_on_failure }}
code:
{% endmacro -%}` },
    testStage: { macroString: `{% macro test_stage(stage_name, continue_on_failure=true, type="sql") %}
---
coalesce_stage_start
name: {{ stage_name }}
type: {{ type }}Test
continue_on_failure: {{ continue_on_failure }}
code:
{% endmacro -%}` },
    getSourceTransform: { macroString: `{%- macro get_source_transform(col) -%}
    {% if col.hashDetails %}
        {{ hash(col.hashDetails.columns, algo=col.hashDetails.algorithm, datatype=col.dataType) }}
    {% elif col.transform | trim != '' %}
        {{ col.transform }}
    {% elif col.sourceColumns[0].node and col.sourceColumns[0].node.name and col.sourceColumns[0].column and col.sourceColumns[0].column.name %}
        "{{ col.sourceColumns[0].node.name }}"."{{ col.sourceColumns[0].column.name }}"
    {% else %}
        NULL
    {% endif %}
{% endmacro -%}` }
};
var RefMacros;
(function (RefMacros) {
    RefMacros["ref"] = "ref";
    RefMacros["refLink"] = "ref_link";
    RefMacros["noLink"] = "ref_no_link";
})(RefMacros = exports.RefMacros || (exports.RefMacros = {}));
/*
Used to provide information to the SComponents.MonacoTooltip, typing exists in SComponents
*/
exports.RefDescriptions = {
    ref: { title: "ref(‘storage_loc’, ‘node_name’)", description: "Resolves the fully qualified node name with a dependency" },
    refLink: { title: "ref_link(‘storage_loc’, ‘node_name’)", description: "Adds a node dependency without resolving to the node name" },
    noLink: { title: "ref_no_link(‘storage_loc’, ‘node_name’)", description: "Resolves to fully qualified node name without a dependency" }
};
/**
 * a helper that converts the way we store macros in state (a dictionary) into
 * the data type that the JSON template renderer requires (a string)
 *
 * @param allMacroDefinitions macro definitions as a dictionary
 * @returns a string, to be sent off to the JSON template renderer
 */
const getAllMacros = (allMacroDefinitions) => {
    let macrosString = '';
    Object.keys(allMacroDefinitions).forEach(macroID => {
        macrosString += `${allMacroDefinitions[macroID].macroString}`;
    });
    return macrosString;
};
exports.getAllMacros = getAllMacros;
//# sourceMappingURL=MacroOperations.js.map