"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.InitializeNewColumnMetadataFromColumnMetadata = exports.GetHydratedMetadataWithNodeMetadata = exports.DeleteColumnMetadata = exports.FormatBulkUpdatedColumnMetadata = exports.BulkSourceMappingError = exports.GetColumnWithUpdatedSourceColumnReference = exports.GetColumnWithUpdatedGenericAttribute = exports.GetColumnWithUpdatedTransform = exports.GetColumnMetadataWithUpdatedColumnGenericAttribute = exports.GetColumnMetadataWithUpdatedColumnSourceIDs = exports.GetColumnMetadataWithUpdatedColumnTransform = exports.IsFirstClassMappingField = exports.EFirstClassNodeMegagridFields = exports.EFirstClassMappingFields = exports.FormatMetadataUpdatesFromParseResult = exports.GetColumnMetadataFromDescTableResult = exports.createNewJoinFromParsedJSON = exports.createColumnMetadataFromParsedJSON = exports.processJSONObj = exports.getDependencyNamesForStep = exports.EditColumnByColumnIndex = exports.EditColumnByColumnCounter = exports.EditAllColumns = exports.getCurrentColumnRef = exports.getSourceColumnRef = exports.getSourceColumnRefData = exports.getSourceStepCountersFromColumnMetadata = exports.getTransformFromColumnMetadata = exports.findColumnByColumnCounter = exports.getCustomSQL = exports.GetSQLTypeSafe = exports.getColumnsForStep = exports.getColumnName = exports.getTableName = exports.getName = exports.GetAllColumnNames = exports.GetEmptyColumnMetadata = exports.CalculateExistingColumns = exports.CalculateNewlyAddedColumns = exports.EMetadataErrors = exports.IsMappingsCollectionEmpty = exports.useStorageMappings = exports.HydratedMetadata = void 0;
const Immer = __importStar(require("immer"));
const Common = __importStar(require("./Common"));
const Runner = __importStar(require("./Runner"));
const RunnerOperations = __importStar(require("./RunnerOperations"));
const StepOperations = __importStar(require("./StepOperations"));
const Logging = __importStar(require("./Logging"));
const AdvancedDeployOperations = __importStar(require("./AdvancedDeployOperations"));
const MetadataHydration = __importStar(require("./MetadataHydration"));
const StepTypes = __importStar(require("./StepTypes"));
const react_redux_1 = require("react-redux");
class HydratedMetadata {
    constructor() {
        this.columns = [];
        this.destinationName = "";
    }
}
exports.HydratedMetadata = HydratedMetadata;
const useStorageMappings = () => {
    const storageMappings = (0, react_redux_1.useSelector)(state => state.mappings);
    return storageMappings;
};
exports.useStorageMappings = useStorageMappings;
/**
 * Helper function to determine if the mappings collection has any values, when empty return true
 * @param mappings
 * @returns
 */
const IsMappingsCollectionEmpty = (mappings) => {
    const length = Object.keys(mappings).length;
    return !!!length;
};
exports.IsMappingsCollectionEmpty = IsMappingsCollectionEmpty;
var EMetadataErrors;
(function (EMetadataErrors) {
    EMetadataErrors["unknownColumn"] = "Unknown Column";
    EMetadataErrors["unknownTable"] = "Unknown Table";
})(EMetadataErrors = exports.EMetadataErrors || (exports.EMetadataErrors = {}));
/**
 * Compares a new set of column metadata against an old set, returning only the new ones
 * @param newColumnMetadata
 * @param currentColumnMetadata
 * @returns array of only the fresh columns' metadata
 */
const CalculateNewlyAddedColumns = (newColumnMetadata, currentColumnMetadata) => {
    const newColumnMetadataResult = {};
    newColumnMetadata.forEach((column) => {
        newColumnMetadataResult[column.name] = column;
    });
    currentColumnMetadata.forEach((column) => {
        delete newColumnMetadataResult[column.name];
    });
    return Object.values(newColumnMetadataResult);
};
exports.CalculateNewlyAddedColumns = CalculateNewlyAddedColumns;
const CalculateExistingColumns = (newColumnMetadata, currentColumnMetadata) => {
    return currentColumnMetadata.filter(currentColumn => {
        return newColumnMetadata.some(newColumn => newColumn.name === currentColumn.name);
    });
};
exports.CalculateExistingColumns = CalculateExistingColumns;
const GetEmptyColumnMetadata = () => {
    return {
        nullable: true,
        defaultValue: "",
        name: "",
        dataType: "",
        transform: "",
        description: "",
        columnReference: {
            stepCounter: "0",
            columnCounter: 0,
        },
        sourceColumnReferences: [
            {
                columnReferences: [
                    {
                        columnCounter: 0,
                        stepCounter: "0",
                    },
                ],
                transform: "",
            },
        ],
        systemColumnType: Runner.ESystemColumnTypes.none,
        keyColumnType: Runner.EKeyColumnTypes.none,
        config: {},
        appliedColumnTests: {},
        acceptedValues: { strictMatch: true, values: [] }
    };
};
exports.GetEmptyColumnMetadata = GetEmptyColumnMetadata;
const GetAllColumnNames = (stepColumns) => {
    return stepColumns.map(col => col.name);
};
exports.GetAllColumnNames = GetAllColumnNames;
const getName = (stepData) => {
    var _a;
    if (!stepData) {
        return null; //empty sourceInputs dont get links to them
    }
    return ((_a = stepData.operation.name) === null || _a === void 0 ? void 0 : _a.length) ? stepData.operation.name : `<Unnamed ${stepData.stepCounter}>`;
};
exports.getName = getName;
const getTableName = (stepData) => {
    if (!stepData)
        return EMetadataErrors.unknownTable;
    let tableName = (0, exports.getName)(stepData);
    return tableName;
};
exports.getTableName = getTableName;
//find and return the column name for a particular step, given a columnCounter, if it exists
const getColumnName = (stepData, columnCounter) => {
    if (!stepData || !stepData.metadata)
        return EMetadataErrors.unknownColumn;
    const column = stepData.metadata.columns.find(data => data.columnReference.columnCounter === columnCounter);
    return (column === null || column === void 0 ? void 0 : column.name) || EMetadataErrors.unknownColumn;
};
exports.getColumnName = getColumnName;
const getColumnsForStep = (allSteps, stepCounter) => {
    return Common.getValueSafe(allSteps[stepCounter], ["operation", "metadata", "columns"], null);
};
exports.getColumnsForStep = getColumnsForStep;
const GetSQLTypeSafe = (allSteps, stepID) => {
    const sqlType = Common.getValueSafe(allSteps, [stepID, "operation", "sqlType"], null);
    if (!sqlType) {
        Common.assert(Logging.LoggingArea.Steps, false, `Error with step ${stepID}: corrupt metadata, invalid sqlType`);
    }
    return sqlType;
};
exports.GetSQLTypeSafe = GetSQLTypeSafe;
const getCustomSQL = (stepData, index) => {
    return Common.getValueSafe(stepData, ["operation", "metadata", "sourceMapping", index, "customSQL", "customSQL"], "");
};
exports.getCustomSQL = getCustomSQL;
const findColumnByColumnCounter = (operation, columnCounter) => {
    var _a;
    if (!operation || !operation.metadata)
        return null;
    return (_a = operation.metadata.columns.find(c => c.columnReference.columnCounter === columnCounter)) !== null && _a !== void 0 ? _a : null;
};
exports.findColumnByColumnCounter = findColumnByColumnCounter;
const getTransformFromColumnMetadata = (columnMetadata, index) => {
    return Common.getValueSafe(columnMetadata, ["sourceColumnReferences", index, "transform"], "");
};
exports.getTransformFromColumnMetadata = getTransformFromColumnMetadata;
const getSourceStepCountersFromColumnMetadata = (columnMetadata) => {
    var _a;
    if (!((_a = columnMetadata.sourceColumnReferences) === null || _a === void 0 ? void 0 : _a.length)) {
        return ["0"];
    }
    const stepCounter = new Set();
    columnMetadata.sourceColumnReferences.forEach((reference, index) => {
        const sourceColumnRef = (0, exports.getSourceColumnRef)(columnMetadata, index);
        const stepCounters = sourceColumnRef.getSourceStepCounters();
        stepCounters.forEach(counter => {
            stepCounter.add(counter);
        });
    });
    return Array.from(stepCounter);
};
exports.getSourceStepCountersFromColumnMetadata = getSourceStepCountersFromColumnMetadata;
/**
 * a safe accessor for source column references.
 * @param columnMetadata the column we're accessing the value of
 * @param index the multisourceIndex
 * @returns the source column ref metadata at the given index. not the class instance.
 */
const getSourceColumnRefData = (columnMetadata, index) => {
    return Common.getValueSafe(columnMetadata, ["sourceColumnReferences", index], { columnReferences: [], transform: "" });
};
exports.getSourceColumnRefData = getSourceColumnRefData;
const getSourceColumnRef = (columnMetadata, index) => {
    const columnRefData = (0, exports.getSourceColumnRefData)(columnMetadata, index);
    return new StepOperations.SourceColumnReference(columnRefData.columnReferences, columnRefData.transform);
};
exports.getSourceColumnRef = getSourceColumnRef;
const getCurrentColumnRef = (col) => {
    const { stepCounter, columnCounter } = col.columnReference;
    return new StepOperations.ColumnReference(stepCounter, columnCounter);
};
exports.getCurrentColumnRef = getCurrentColumnRef;
const EditColumnInternal = (columns, predicateFunc, columnEditFunc) => {
    return Immer.produce(columns, (draftState) => {
        draftState.forEach((col, index) => {
            if (predicateFunc(col, index)) {
                columnEditFunc(col);
            }
        });
    });
};
const EditAllColumns = (columns, columnEditFunc) => {
    return EditColumnInternal(columns, () => true, columnEditFunc);
};
exports.EditAllColumns = EditAllColumns;
const EditColumnByColumnCounter = (columns, columnCounter, columnEditFunc) => {
    return EditColumnInternal(columns, (column => column.columnReference.columnCounter === columnCounter), columnEditFunc);
};
exports.EditColumnByColumnCounter = EditColumnByColumnCounter;
const EditColumnByColumnIndex = (columns, columnIndex, columnEditFunc) => {
    Common.assert(Logging.LoggingArea.MetadataOperations, (columnIndex < columns.length && columnIndex > -1), `Received invalid columnIndex of ${columnIndex}. Acceptable range between 0-${columns.length - 1}`);
    return EditColumnInternal(columns, ((c, index) => index === columnIndex), columnEditFunc);
};
exports.EditColumnByColumnIndex = EditColumnByColumnIndex;
/**
 * Dupe function as GraphOperations.getDependenciesForStep, except returns RefLink[] instead of stepCounters
 * Need this version because if a stored dependency is returned as a stepCounter, then I lose access to it's name
 * Which is what I need to check before allowing a rename
 * Refactor to return both? RefLink extended where { nodeName, locationName, stepCounter }[] is the return type?
 */
const getDependencyNamesForStep = (steps, stepData, index) => {
    const specifiedIndex = !(index === null);
    const dependencies = [];
    if (stepData === null)
        return dependencies;
    /*
     * iterate through the stepData.operation.dependencies and add them to allDependencies before converting to stepCounters
     * sources don't have sourceMaps
     */
    if (stepData.operation.sqlType !== Runner.ESQLType.source) {
        stepData.operation.metadata.sourceMapping.forEach((sourceMap, currIndex) => {
            // control flow to account for generating joins for individual sources of a multisource
            if (specifiedIndex && index === currIndex) { // runs once
                sourceMap.dependencies.forEach(dependency => {
                    dependencies.push(dependency);
                });
            }
            else if (!specifiedIndex) { // runs every time
                sourceMap.dependencies.forEach(dependency => {
                    dependencies.push(dependency);
                });
            }
        });
        stepData.operation.metadata.columns.forEach(col => {
            let dependenciesToAdd = [];
            if (!specifiedIndex) {
                dependenciesToAdd = (0, exports.getSourceStepCountersFromColumnMetadata)(col);
            }
            else {
                const sourceColumnRef = (0, exports.getSourceColumnRef)(col, index);
                dependenciesToAdd = sourceColumnRef.getSourceStepCounters();
            }
            dependenciesToAdd.forEach(dependency => {
                const step = steps[dependency];
                if (step && step.operation) {
                    dependencies.push({ nodeName: step.operation.name, locationName: step.operation.locationName });
                }
            });
        });
    }
    return dependencies;
};
exports.getDependencyNamesForStep = getDependencyNamesForStep;
const processJSONObj = (currentValue, currentKeyPath, alias) => {
    console.assert(currentKeyPath.length >= 1); // current key path needs to include the source column name/alias as the first value - the length should always be at least 1
    switch (typeof currentValue) {
        case ("string"):
        case ("number"):
        case ("boolean"):
            // treat null values as primitives and create a new column
            return createNewColumnDataFromJSON(currentValue, currentKeyPath, alias);
        case ("object"):
            if (currentValue === null) {
                return createNewColumnDataFromJSON("null", currentKeyPath, alias);
            }
            else {
                let parsedData = { newColumns: [], joins: [] };
                if (currentValue.length === undefined) {
                    for (let key in currentValue) {
                        // create columns for each key in the object
                        const result = (0, exports.processJSONObj)(currentValue[key], [...currentKeyPath, key], alias);
                        parsedData.newColumns = parsedData.newColumns.concat(result.newColumns);
                        parsedData.joins = parsedData.joins.concat(result.joins);
                    }
                }
                else {
                    if (currentValue.length !== 0) {
                        // call flatten, update join
                        const newAlias = currentKeyPath.join("_");
                        let result;
                        if (isPrimitive(currentValue[0])) { // handle array of primitives case here
                            result = createNewColumnDataFromJSON(currentValue, currentKeyPath, newAlias);
                        }
                        else {
                            result = (0, exports.processJSONObj)(currentValue[0], currentKeyPath, newAlias);
                        }
                        parsedData.newColumns = parsedData.newColumns.concat(result.newColumns);
                        parsedData.joins = parsedData.joins.concat(result.joins, createParsedJSONJoin(currentValue, currentKeyPath, alias, newAlias));
                    }
                }
                return parsedData;
            }
        default:
            console.error("invalid type", currentValue, currentKeyPath);
            return { newColumns: [], joins: [] };
    }
};
exports.processJSONObj = processJSONObj;
const createColumnMetadataFromParsedJSON = (parsedJSON, teamID, currentColumnLocation, sourceColumnLocation, stepData) => {
    return StepOperations.getNextColumnCounter(parsedJSON.newColumns.length, teamID).then(nextColumnCounter => {
        // only filter out the column that you used parse
        const filteredStepColumns = stepData.operation.metadata.columns.filter(col => col.columnReference.columnCounter !== currentColumnLocation.columnCounter);
        const newColumns = parsedJSON.newColumns.map((col, index) => {
            let columnName;
            if (!col.name) {
                const columnData = (0, exports.findColumnByColumnCounter)(stepData.operation, currentColumnLocation.columnCounter);
                columnName = columnData ? columnData.name : "Unknown column";
            }
            else {
                columnName = col.name;
            }
            return {
                name: columnName,
                dataType: col.type,
                nullable: true,
                columnReference: {
                    stepCounter: currentColumnLocation.stepCounter,
                    columnCounter: index + nextColumnCounter
                },
                description: "",
                sourceColumnReferences: [{
                        transform: col.transform,
                        columnReferences: [sourceColumnLocation.toJSON()] // serialized, sourceColumnLocation on its own is a class instance
                    }],
                config: {},
                appliedColumnTests: {},
                acceptedValues: { strictMatch: true, values: [] }
            };
        });
        return [...filteredStepColumns, ...newColumns];
    });
};
exports.createColumnMetadataFromParsedJSON = createColumnMetadataFromParsedJSON;
const createNewJoinFromParsedJSON = (parsedJSON, oldJoin) => {
    let newJoinValues = [oldJoin];
    /*
     *in scenarios with nested arrays, you need to be able to correctly reference the parent flattened table's alias,
     *which from the algo consistently get created and added to this array after their children. we call reverse here to
     *make sure the flatten functions for parent tables are appended to the join before their children
     */
    let reversedNewJoins = parsedJSON.joins.reverse();
    reversedNewJoins.forEach(join => newJoinValues.push(join));
    return newJoinValues.join(", \n");
};
exports.createNewJoinFromParsedJSON = createNewJoinFromParsedJSON;
/////// TRANSFORM CREATION
const createParsedJSONTransform = (obj, currentKeyPath, alias) => {
    const sourceName = currentKeyPath[0];
    const dataType = getFormattedDatatype(obj);
    const valueIdentifier = createValueIdentifier(obj, currentKeyPath, alias);
    if (hasAlias(alias)) {
        return `"${alias}".value${valueIdentifier}::${dataType}`;
    }
    return `"${sourceName}"${valueIdentifier}::${dataType}`;
};
/*
 * for creating the accessor part of the transform - returns similar syntax
 * to below for join value identifiers, but with slightly different control flow
 */
const createValueIdentifier = (obj, currentKeyPath, alias) => {
    const pathForName = currentKeyPath.slice(1);
    // value should be empty string if obj is an array of primitives
    if (isArray(obj)) {
        return ``;
    }
    // if currently under an alias, value identifier should exclude the key path already described in the alias
    if (hasAlias(alias)) {
        return createValueIdentifierWithAlias(currentKeyPath, alias);
    }
    return `:${pathForName.map(key => `"${key}"`).join(".")}`;
};
///////// JOIN CREATION
const createParsedJSONJoin = (obj, keyPath, alias, newAlias) => {
    const sourceName = keyPath[0];
    if (hasAlias(alias)) {
        return `table(flatten("${alias}".value${createValueIdentifierForJoin(keyPath, alias)})) "${newAlias}"`;
    }
    return `table(flatten("${sourceName}"${createValueIdentifierForJoin(keyPath, alias)})) "${newAlias}"`;
};
const createValueIdentifierForJoin = (keyPath, alias) => {
    if (hasAlias(alias)) {
        return createValueIdentifierWithAlias(keyPath, alias);
    }
    const identifierKeyPath = keyPath.slice(1);
    return identifierKeyPath.length ? `:${identifierKeyPath.map(key => `"${key}"`).join(".")}` : "";
};
///////// COLUMN CREATION
// column name is all values from the current key path except the first joined together
const createParsedColumnName = (stringKeyPath) => {
    const pathForName = stringKeyPath.slice(1);
    if (pathForName.length === 1) {
        return pathForName[0];
    }
    return pathForName.join("_");
};
// generates the data used to create actual column metadata
const createNewColumnDataFromJSON = (obj, currentKeyPath, alias) => {
    return {
        joins: [],
        newColumns: [{
                name: createParsedColumnName(currentKeyPath),
                transform: createParsedJSONTransform(obj, currentKeyPath, alias),
                type: getFormattedDatatype(obj)
            }]
    };
};
// JSON HELPERS
const isArray = (value) => {
    return Array.isArray(value);
};
const isPrimitive = (value) => {
    const primitiveTypes = [
        "string",
        "number",
        "boolean"
    ];
    return primitiveTypes.includes(typeof value) || value === null;
};
const getFormattedDatatype = (value) => {
    return isArray(value) ?
        (typeof value[0]).toUpperCase() :
        (typeof value).toUpperCase();
};
const hasAlias = (alias) => {
    return !(alias === null);
};
const createValueIdentifierWithAlias = (keyPath, alias) => {
    const aliasLength = alias.split("_").length;
    return keyPath.length ? `:${keyPath.slice(aliasLength).map(key => `"${key}"`).join(".")}` : "";
};
//////////
const GetColumnMetadataFromDescTableResult = (queryResult, teamID, stepCounter) => {
    return StepOperations.getNextColumnCounter(queryResult.fields.length, teamID).then(columnCounter => {
        return queryResult.rows.map((row, index) => {
            return {
                name: row.values[0],
                dataType: row.values[1],
                transform: "",
                nullable: row.values[3] === "Y" ? true : false,
                defaultValue: "",
                description: "",
                columnReference: {
                    stepCounter: stepCounter,
                    columnCounter: columnCounter + index
                },
                sourceColumnReferences: [{
                        transform: "",
                        columnReferences: []
                    }],
                config: {},
                appliedColumnTests: {},
                acceptedValues: { strictMatch: true, values: [] }
            };
        });
    });
};
exports.GetColumnMetadataFromDescTableResult = GetColumnMetadataFromDescTableResult;
const FormatMetadataUpdatesFromParseResult = (allSteps, teamID, parseResult, stepCounter) => {
    let updates = [];
    const currentStepData = allSteps[stepCounter];
    // 1) CALCULATE PRE EXISTING AND NEWLY ADDED COLUMNS
    // format metadata for CalculateNewlyAddedColumns
    const formattedParsedColumnMetadata = parseResult.columnMetadata.map(column => {
        return {
            name: column.name,
            dataType: column.dataType,
            nullable: false,
            columnReference: {
                stepCounter: stepCounter,
                columnCounter: 0
            },
            description: "",
            sourceColumnReferences: [
                {
                    columnReferences: [],
                    transform: column.transform
                }
            ],
            appliedColumnTests: {},
            acceptedValues: {}
        };
    });
    // perform calculations
    const currentStepColumns = Common.getValueSafe(currentStepData, ["operation", "metadata", "columns"], []);
    const newlyAddedColumns = (0, exports.CalculateNewlyAddedColumns)(formattedParsedColumnMetadata, currentStepColumns);
    const existingColumns = (0, exports.CalculateExistingColumns)(formattedParsedColumnMetadata, currentStepColumns);
    // 2) ASSIGN COLUMN COUNTERS TO THE NEWLY PARSED COLUMNS
    let formatNewlyParsedColumns;
    // update newly added columns with their column counters
    if (newlyAddedColumns.length) {
        formatNewlyParsedColumns = () => StepOperations.getNextColumnCounter(newlyAddedColumns.length, teamID)
            .then(nextColumnCounter => {
            return newlyAddedColumns.map((column, index) => {
                return Object.assign(Object.assign({}, column), { columnReference: {
                        stepCounter: stepCounter,
                        columnCounter: index + nextColumnCounter
                    } });
            });
        });
    }
    else {
        // if there are no newly added columns, just resolve with an empty array
        formatNewlyParsedColumns = () => Promise.resolve([]);
    }
    // 3) UPDATE ANY PRE EXISTING COLUMNS AND RETURN ALL UPDATED COLUMN METADATA
    const getNewColumnMetadata = () => formatNewlyParsedColumns()
        .then(newlyParsedColumns => {
        const updatedColumns = Immer.produce(existingColumns, draftState => {
            draftState.forEach((draftColumn, index) => {
                // match columns to update by name
                const columnMatch = parseResult.columnMetadata.find(parsedColumn => parsedColumn.name === draftColumn.name);
                // when a match is found, overwrite its datatype and transform
                if (columnMatch) {
                    draftState[index].dataType = columnMatch.dataType;
                    if (!draftState[index].sourceColumnReferences[0]) {
                        draftState[index].sourceColumnReferences[0] = {
                            columnReferences: [],
                            transform: columnMatch.transform
                        };
                    }
                    else {
                        draftState[index].sourceColumnReferences[0].transform = columnMatch.transform;
                    }
                }
            });
        });
        return [...updatedColumns, ...newlyParsedColumns];
    })
        .catch(error => {
        console.error("error formatting new column metadata:", error);
    });
    // 4) FORMAT FINAL UPDATE
    return getNewColumnMetadata().then(updatedColumns => {
        // safe to hardcode to the first index here - custom sql is not compatible with multisource for now
        const sourceMap = Runner.getSourceMapFromOperationSafe(currentStepData.operation, 0);
        const updatedSourceMap = Runner.getUpdatedSourceMapJoin(sourceMap, parseResult.joinString);
        updates.push({
            key: "metadata",
            value: {
                cteString: parseResult.cteString,
                columns: updatedColumns,
                // edge case: disabling multisource on a node and then enabling the "unstructured view" - as it stands this would override any source mapping
                sourceMapping: [updatedSourceMap]
            },
            stepCounter
        });
        return updates;
    });
};
exports.FormatMetadataUpdatesFromParseResult = FormatMetadataUpdatesFromParseResult;
var EFirstClassMappingFields;
(function (EFirstClassMappingFields) {
    EFirstClassMappingFields["source"] = "source";
    EFirstClassMappingFields["dataType"] = "dataType";
    EFirstClassMappingFields["description"] = "description";
    EFirstClassMappingFields["nullable"] = "nullable";
    EFirstClassMappingFields["defaultValue"] = "defaultValue";
    EFirstClassMappingFields["name"] = "name";
    EFirstClassMappingFields["transform"] = "transform";
})(EFirstClassMappingFields = exports.EFirstClassMappingFields || (exports.EFirstClassMappingFields = {}));
var EFirstClassNodeMegagridFields;
(function (EFirstClassNodeMegagridFields) {
    EFirstClassNodeMegagridFields["nodeName"] = "nodeName";
    EFirstClassNodeMegagridFields["runStatus"] = "runStatus";
})(EFirstClassNodeMegagridFields = exports.EFirstClassNodeMegagridFields || (exports.EFirstClassNodeMegagridFields = {}));
const IsFirstClassMappingField = (columnField) => {
    return Object.values(EFirstClassMappingFields).includes(columnField);
};
exports.IsFirstClassMappingField = IsFirstClassMappingField;
// STEP COLUMN UPDATE HELPERS
/**
 *
 * @param allSteps
 * @param stepID id of the node to update
 * @param columnIndex index of the column to update
 * @param multisourceIndex index of the sourceColumnReference to update
 * @param newTransform
 * @returns all column metadata for the node at the given stepID, with the column at the given
 * column index updated with the given transform
 */
const GetColumnMetadataWithUpdatedColumnTransform = (allSteps, stepID, columnIndex, multisourceIndex, newTransform) => {
    const stepColumns = (0, exports.getColumnsForStep)(allSteps, stepID);
    // create a copy we can modify
    const columnMetadataCopyForUpdate = Common.CreateDeepClone(stepColumns);
    // hydrate any uses of {{SRC}}, {{SRC_COL}}, etc.
    const processedTransform = Runner.hydrateTransform(allSteps, columnMetadataCopyForUpdate[columnIndex], newTransform, multisourceIndex);
    const updatedColumn = (0, exports.GetColumnWithUpdatedTransform)(columnMetadataCopyForUpdate[columnIndex], processedTransform, multisourceIndex);
    columnMetadataCopyForUpdate[columnIndex] = updatedColumn;
    return columnMetadataCopyForUpdate;
};
exports.GetColumnMetadataWithUpdatedColumnTransform = GetColumnMetadataWithUpdatedColumnTransform;
/**
 *
 * @param allSteps
 * @param stepID id of the node to update
 * @param columnIndex index of the column to update
 * @param multisourceIndex index of the sourceColumnReference to be updated
 * @param newColumnReference
 * @returns all column metadata for the node at the given stepID, with the column at the given
 * column index updated with the given column reference
 */
const GetColumnMetadataWithUpdatedColumnSourceIDs = (allSteps, stepID, columnIndex, multisourceIndex, newColumnReference) => {
    const currentStepColumns = (0, exports.getColumnsForStep)(allSteps, stepID);
    const columnMetadataCopyForUpdate = Common.CreateDeepClone(currentStepColumns);
    columnMetadataCopyForUpdate[columnIndex] = (0, exports.GetColumnWithUpdatedSourceColumnReference)(currentStepColumns[columnIndex], multisourceIndex, newColumnReference);
    return columnMetadataCopyForUpdate;
};
exports.GetColumnMetadataWithUpdatedColumnSourceIDs = GetColumnMetadataWithUpdatedColumnSourceIDs;
/**
 *
 * @param allSteps
 * @param stepID id of the node to update
 * @param columnIndex index of the column to update
 * @param field the name of the attribute to update
 * @param newValue the new value for the attribute
 * @returns a full node's column metadata with the column at the column index updated with
 * the given field and attribute
 */
const GetColumnMetadataWithUpdatedColumnGenericAttribute = (allSteps, stepID, columnIndex, field, newValue) => {
    const currentStepColumns = (0, exports.getColumnsForStep)(allSteps, stepID);
    const columnMetadataCopyForUpdate = Common.CreateDeepClone(currentStepColumns);
    const updatedColumn = (0, exports.GetColumnWithUpdatedGenericAttribute)(columnMetadataCopyForUpdate[columnIndex], field, newValue);
    columnMetadataCopyForUpdate[columnIndex] = updatedColumn;
    return columnMetadataCopyForUpdate;
};
exports.GetColumnMetadataWithUpdatedColumnGenericAttribute = GetColumnMetadataWithUpdatedColumnGenericAttribute;
const GetColumnWithUpdatedTransform = (column, newTransform, multisourceIndex) => {
    Common.assert(Logging.LoggingArea.MetadataOperations, !!column, `GetColumnWithUpdatedTransform was a falsy value: ${column}`);
    const columnMetadataCopyForUpdate = Common.CreateDeepClone(column);
    if (!column.sourceColumnReferences || !column.sourceColumnReferences[multisourceIndex]) {
        const newSourceColumnRef = {
            columnReferences: [],
            transform: newTransform
        };
        columnMetadataCopyForUpdate.sourceColumnReferences[multisourceIndex] = newSourceColumnRef;
    }
    else {
        columnMetadataCopyForUpdate.sourceColumnReferences[multisourceIndex].transform = newTransform;
    }
    return columnMetadataCopyForUpdate;
};
exports.GetColumnWithUpdatedTransform = GetColumnWithUpdatedTransform;
/**
 *
 * @param column column to update
 * @param field the attribute name
 * @param value the new value
 * @returns a column with the updated attribute
 */
const GetColumnWithUpdatedGenericAttribute = (column, field, value) => {
    const columnMetadataCopyForUpdate = Common.CreateDeepClone(column);
    /**
     * we still determine whether or not to update the config or root level
     * here, in the same way that we would with config menu items
     */
    if ((0, exports.IsFirstClassMappingField)(field)) {
        columnMetadataCopyForUpdate[field] = value;
    }
    else {
        columnMetadataCopyForUpdate.config = Object.assign(Object.assign({}, columnMetadataCopyForUpdate.config), { [field]: value });
    }
    return columnMetadataCopyForUpdate;
};
exports.GetColumnWithUpdatedGenericAttribute = GetColumnWithUpdatedGenericAttribute;
/**
 *
 * @param column column to update
 * @param multisourceIndex index of sourceColumnReference to update
 * @param newColumnReference either a new column reference, or null. if null, this clears the current column reference.
 * serves the purpose of handling unmapping from its current source
 * @returns a column with an updated source column reference
 */
const GetColumnWithUpdatedSourceColumnReference = (column, multisourceIndex, newColumnReference) => {
    const updatedColumnCopy = Common.CreateDeepClone(column);
    Common.assert(Logging.LoggingArea.MetadataOperations, !!column && !!column.sourceColumnReferences, `Column ${column.name} is missing sourceColumnReference attribute`);
    /**
     * placeholder until we determine how a customer can get into a state where they have a
     * column without a source column reference. see CD-3414
     */
    if (!updatedColumnCopy.sourceColumnReferences[multisourceIndex]) {
        updatedColumnCopy.sourceColumnReferences[multisourceIndex] = StepOperations.GetEmptySourceColumnReference();
    }
    if (newColumnReference === null || !newColumnReference.columnCounter || !newColumnReference.stepCounter) {
        // unmap the source column
        updatedColumnCopy.sourceColumnReferences[multisourceIndex].columnReferences = [];
    }
    else {
        // update the source column
        updatedColumnCopy.sourceColumnReferences[multisourceIndex].columnReferences[0] = newColumnReference;
    }
    return updatedColumnCopy;
};
exports.GetColumnWithUpdatedSourceColumnReference = GetColumnWithUpdatedSourceColumnReference;
exports.BulkSourceMappingError = "Error finding a valid source column match";
/**
 *
 * @param columnMetadata array of columns to alter
 * @param columnIndeces indeces of columns to update within the array
 * @param columnFieldsToUpdate the fields of columns to update
 * @param modelViewerState named this for clarity, as modelViewerState is the dictionary
 * we track our changes in - just a dictionary whose keys are the columnFields, and attributes are
 * their new values
 */
const FormatBulkUpdatedColumnMetadata = (columnMetadata, columnIndeces, columnFieldsToUpdate, modelViewerState, allSteps) => {
    let bulkUpdateErrors = [];
    const columnMetadataCopyForUpdate = Common.CreateDeepClone(columnMetadata);
    // this ensures sourceIDs get updated first, preventing a small bug when hydrating a transform with a token
    const orderedColumnFields = ReorderColumnFieldsToUpdateByPriority(columnFieldsToUpdate);
    columnIndeces.forEach(index => {
        orderedColumnFields.forEach(field => {
            switch (field) {
                case EFirstClassMappingFields.transform:
                    const processedTransform = Runner.hydrateTransform(allSteps, columnMetadataCopyForUpdate[index], modelViewerState[field], modelViewerState.currentSource);
                    columnMetadataCopyForUpdate[index] = (0, exports.GetColumnWithUpdatedTransform)(columnMetadataCopyForUpdate[index], processedTransform, modelViewerState.currentSource);
                    break;
                case EFirstClassMappingFields.source:
                    if (modelViewerState[field].length === 1) {
                        const sourceColumns = (0, exports.getColumnsForStep)(allSteps, modelViewerState[field][0]);
                        const mappedColumnResult = StepOperations.FindSourceColumnMatchByName(columnMetadataCopyForUpdate[index], sourceColumns, modelViewerState.currentSource, allSteps);
                        // if no match found, set error
                        if (mappedColumnResult === null) {
                            bulkUpdateErrors.push(exports.BulkSourceMappingError);
                        }
                        else {
                            columnMetadataCopyForUpdate[index] = (0, exports.GetColumnWithUpdatedSourceColumnReference)(columnMetadataCopyForUpdate[index], modelViewerState.currentSource, mappedColumnResult.columnReference);
                        }
                    }
                    else {
                        const valueForUpdate = modelViewerState[field].length ? modelViewerState[field] : [0, 0];
                        const formattedColumnReference = {
                            stepCounter: valueForUpdate[0],
                            columnCounter: valueForUpdate[1]
                        };
                        columnMetadataCopyForUpdate[index] = (0, exports.GetColumnWithUpdatedSourceColumnReference)(columnMetadataCopyForUpdate[index], modelViewerState.currentSource, formattedColumnReference);
                    }
                    break;
                default:
                    const updatedColumn = (0, exports.GetColumnWithUpdatedGenericAttribute)(columnMetadataCopyForUpdate[index], field, modelViewerState[field]);
                    columnMetadataCopyForUpdate[index] = updatedColumn;
            }
        });
    });
    return {
        updatedColumns: columnMetadataCopyForUpdate,
        errors: bulkUpdateErrors
    };
};
exports.FormatBulkUpdatedColumnMetadata = FormatBulkUpdatedColumnMetadata;
/**
 * @param columnFields
 * @returns a reordered array that ensures sourceIDs are first
 */
const ReorderColumnFieldsToUpdateByPriority = (columnFields) => {
    return Object.values(EFirstClassMappingFields).filter(field => columnFields.includes(field));
};
const DeleteColumnMetadata = (columnMetadata, columnIDsToDelete) => {
    const remainingColumns = columnMetadata.filter(column => !columnIDsToDelete.includes(column.columnReference.columnCounter));
    const remainingColumnsWithUpdatedHashes = remainingColumns.map(column => {
        var _a;
        if ((_a = column.hashedColumns) === null || _a === void 0 ? void 0 : _a.some(hashedColumn => columnIDsToDelete.includes(hashedColumn.columnCounter))) {
            const newHashedColumns = column.hashedColumns.filter(hashedColumn => {
                return !columnIDsToDelete.includes(hashedColumn.columnCounter);
            });
            column.hashedColumns = newHashedColumns;
            return column;
        }
        return column;
    });
    return remainingColumnsWithUpdatedHashes;
};
exports.DeleteColumnMetadata = DeleteColumnMetadata;
/**
 * Returns hydrated metadata embedded in node metadata for both standard and advanced deploy strategies.
 * Currently only the "create" operation type will check whether the node is using an advanced deploy strategy
 * For now, all other operation types will always return standard hydrated metadata
 */
const GetHydratedMetadataWithNodeMetadata = (stepID, state, operationType, specifiedIndex, metadataOverride, runTimeParameters) => {
    const steps = state.steps;
    const stepTypesYaml = state.stepTypesYaml;
    const workspaces = state.workspaces;
    const nodeMetadata = steps[stepID].operation;
    const currentWorkspaceData = workspaces.workspaces[workspaces.currentWorkspace];
    const mapping = currentWorkspaceData.mappings || {};
    StepTypes.ThrowIfStepTypeDoesntExist(stepTypesYaml, steps[stepID].operation.sqlType);
    Common.assert(Logging.LoggingArea.AdvancedDeploy, (operationType in RunnerOperations.OperationTypes), `Operation type ${operationType} is not a valid operation`);
    const logger = Logging.GetLogger(Logging.LoggingArea.Deploy);
    if (!!metadataOverride) {
        logger.info(`using metadataOverride for metadata hydration for step ${stepID} with operation type ${operationType}`);
        return metadataOverride;
    }
    // currently advanced deploy only utilizes the create script
    if (operationType === RunnerOperations.OperationTypes.create) {
        const isStepAdvancedDeploy = AdvancedDeployOperations.GetStepDeployStrategyFromStepTypesYAML(stepID, steps, stepTypesYaml) === AdvancedDeployOperations.EDeployStrategy.advanced;
        if (isStepAdvancedDeploy) {
            logger.info(`getting advanced deploy metadata for step ${stepID} with operation type ${operationType}`);
            return AdvancedDeployOperations.GetAddTableAdvancedHydratedMetadata(nodeMetadata, stepID, specifiedIndex, mapping, steps, stepTypesYaml, workspaces, runTimeParameters);
        }
    }
    logger.info(`getting standard deploy metadata for step ${stepID} with operation type ${operationType}`);
    try {
        const hydratedMetadata = MetadataHydration.hydrateMetadata(nodeMetadata, stepID, steps, stepTypesYaml, workspaces, specifiedIndex, runTimeParameters);
        return Object.assign(Object.assign({}, hydratedMetadata), { operationName: nodeMetadata.name, destinationName: nodeMetadata.locationName, mapping });
    }
    catch (error) {
        // catching just to assert in this specific case
        Common.assert(Logging.LoggingArea.MetadataHydration, false, `Error in GetHydratedMetadataWithNodeMetadata: ${error.message}`);
        throw error;
    }
};
exports.GetHydratedMetadataWithNodeMetadata = GetHydratedMetadataWithNodeMetadata;
const InitializeNewColumnMetadataFromColumnMetadata = (originalColumnMetadata, newName, newColumnReference, newSourceColumnReferences) => {
    /**
     * sourceColumnReferencesToUse value is spread into the object below
     * Not all cols should have the sourceColumnReferences field
     * if sourceColumnReferences is null, then an empty object will be spread into the object below, resulting in no new field
     * otherwise, it will add the field sourceColumnReferences with the appropriate value
     */
    const sourceColumnReferencesToUse = newSourceColumnReferences ? { sourceColumnReferences: newSourceColumnReferences } : {};
    const { nullable, dataType, description } = originalColumnMetadata;
    return Object.assign({ name: newName, nullable,
        dataType,
        description, appliedColumnTests: {}, acceptedValues: { strictMatch: false, values: [] }, columnReference: newColumnReference }, sourceColumnReferencesToUse);
};
exports.InitializeNewColumnMetadataFromColumnMetadata = InitializeNewColumnMetadataFromColumnMetadata;
//# sourceMappingURL=MetadataOperations.js.map