"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApplyBulkUpdate = exports.PreviewBulkUpdate = exports.ENodeAttributes = void 0;
const Common = __importStar(require("./Common"));
const CommonOperations = __importStar(require("./CommonOperations"));
const Logging = __importStar(require("./Logging"));
const Firebase = __importStar(require("./Firebase"));
/* Indentation size for this file is 2 */
// TYPES AND CLASSES
var ENodeAttributes;
(function (ENodeAttributes) {
    ENodeAttributes["NodeType"] = "nodeType";
    ENodeAttributes["StorageLocation"] = "storageLocation";
})(ENodeAttributes = exports.ENodeAttributes || (exports.ENodeAttributes = {}));
class LocationUpdate {
    // future property
    // applyToSuccessors:boolean
    constructor(stepID, newLocationID, newLocationName) {
        this.stepID = stepID;
        this.newLocationID = newLocationID;
        this.newLocationName = newLocationName;
    }
    apply(allStepsData) {
        const updatedData = Object.assign({}, allStepsData[this.stepID]);
        updatedData.operation.locationID = this.newLocationID;
        updatedData.operation.locationName = this.newLocationName;
        return { [this.stepID]: updatedData };
    }
}
class NodeTypeUpdate {
    // future property
    // shouldUpdateDownstreamRefs: boolean
    constructor(stepID, newNodeTypeID) {
        this.stepID = stepID;
        this.newNodeTypeID = newNodeTypeID;
    }
    apply(allStepsData) {
        const updatedData = Object.assign({}, allStepsData[this.stepID]);
        updatedData.operation.sqlType = this.newNodeTypeID;
        return { [this.stepID]: updatedData };
    }
}
// MAIN EXPORTED FUNCTIONS 
/* PreviewBulkUpdate will format TNodeBulkUpdatesArray and oldValuesArray to return an array of combinedValueObject as row data for the preview table  */
const PreviewBulkUpdate = (allStepsData, selectedStepIDs, selectedAttributes, nodeTypes) => {
    // new values
    const bulkUpdatesArray = CreateBulkUpdatesArray(allStepsData, selectedStepIDs, selectedAttributes);
    // old values
    const oldValuesArray = CreateOldValuesArray(allStepsData, selectedStepIDs, selectedAttributes);
    // check if oldValuesArray has the same length of selecetedStepIDs
    Common.assert(Logging.LoggingArea.UI, oldValuesArray.length === selectedStepIDs.length, `Error in oldValuesArray, lengths don't match. Should have ${selectedStepIDs.length} but got ${oldValuesArray.length}`);
    // combined object with new and old values
    const combinedValuesArray = oldValuesArray.map((oldValueObject) => {
        const updatesWithSameStepID = bulkUpdatesArray.filter((update) => update.stepID === oldValueObject.stepID);
        updatesWithSameStepID.forEach((update) => {
            if (update.hasOwnProperty("newNodeTypeID")) {
                update.newNodeTypeName = nodeTypes[update.newNodeTypeID].name;
                update.oldNodeTypeName = nodeTypes[oldValueObject.oldNodeTypeID].name;
            }
            Object.assign(oldValueObject, update);
        });
        return oldValueObject;
    });
    return combinedValuesArray;
};
exports.PreviewBulkUpdate = PreviewBulkUpdate;
/* ApplyBulkUpdate will kick off FirestoreBatchBarrel */
const ApplyBulkUpdate = (allStepsData, selectedStepIDs, selectedAttributes, teamID, workspaceID) => {
    const finalUpdateResult = GatherUpdates(allStepsData, selectedStepIDs, selectedAttributes);
    let batchBarrelFuncArray = [];
    finalUpdateResult.stepsThatHaveChanged.forEach(stepID => {
        const docRef = CommonOperations.getDocumentRef(teamID, workspaceID, "steps").doc(stepID);
        const value = finalUpdateResult.finalStepState[stepID].operation;
        const func = Common.BatchSetMergeFirestoreDoc(docRef, value);
        batchBarrelFuncArray.push(func);
    });
    return Common.FirestoreBatchBarrel(batchBarrelFuncArray, 500, Firebase.getFirestore());
};
exports.ApplyBulkUpdate = ApplyBulkUpdate;
// HELPER FUNCTIONS
/* CreateBulkUpdatesArray will return all seletcted step with all attribute updates */
const CreateBulkUpdatesArray = (allStepsData, selectedStepIDs, selectedAttributes) => {
    let resultArray = [];
    selectedAttributes.forEach(attributeUpdate => {
        switch (attributeUpdate.attributeType) {
            case ENodeAttributes.NodeType:
                selectedStepIDs.forEach(stepID => {
                    if (!(allStepsData[stepID].operation.sqlType === "Source")) { // only update nodeType for non-source nodes
                        const nodeTypeUpdate = new NodeTypeUpdate(stepID, attributeUpdate.newValue.sqlType);
                        resultArray.push(nodeTypeUpdate);
                    }
                });
                break;
            case ENodeAttributes.StorageLocation:
                selectedStepIDs.forEach(stepID => {
                    const locationUpdate = new LocationUpdate(stepID, attributeUpdate.newValue.locationID, attributeUpdate.newValue.locationName);
                    resultArray.push(locationUpdate);
                });
                break;
            default: break;
        }
    });
    return resultArray;
};
/* CreateOldValuesArray will have the length of selectedStepIDs, meaning each object in the array is one selected node with old values for all selected attributes  */
const CreateOldValuesArray = (allStepsData, selectedStepIDs, selectedAttributes) => {
    return selectedStepIDs.map((stepID) => {
        return selectedAttributes.reduce((accumulator, attribute) => {
            switch (attribute.attributeType) {
                case ENodeAttributes.NodeType:
                    accumulator["oldNodeTypeID"] = allStepsData[stepID].operation.sqlType;
                    accumulator.stepName = allStepsData[stepID].operation.name;
                    break;
                case ENodeAttributes.StorageLocation:
                    accumulator["oldLocationID"] = allStepsData[stepID].operation.locationID;
                    accumulator["oldLocationName"] = allStepsData[stepID].operation.locationName;
                    accumulator.stepName = allStepsData[stepID].operation.name;
                    break;
                default:
                    Common.assert(Logging.LoggingArea.UI, false, `Invalid node attribute in CreateOldValuesArray: ${attribute.attributeType}`);
            }
            return accumulator;
        }, { stepID: stepID, stepName: "" });
    });
};
/* GatherUpdates essentially takes BulkUpdatesArray and returns a blob that's used for ApplyBulkUpdate */
const GatherUpdates = (allStepsData, selectedStepIDs, selectedAttributes) => {
    const bulkUpdates = CreateBulkUpdatesArray(allStepsData, selectedStepIDs, selectedAttributes);
    const bulkUpdatesStepIDs = Array.from(new Set(bulkUpdates.map(update => update.stepID)));
    let interimStepState = allStepsData;
    /* as bulkUpdates array contains every step with every attribute update, we call apply method to update
       allStepsData and accumulate such updates to make sure the final allStepsData has everything
    */
    bulkUpdates.forEach(update => {
        const updatedStep = update.apply(interimStepState);
        interimStepState = Object.assign(Object.assign({}, interimStepState), updatedStep);
    });
    return { stepsThatHaveChanged: bulkUpdatesStepIDs, finalStepState: interimStepState };
};
//# sourceMappingURL=NodeBulkEditor.js.map