"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidateProjectAsRunType = exports.GetDefaultProjectName = exports.GetQualifiedProjectNameFromString = exports.listenProjectChangeCb = exports.RProjectType = exports.defaultProjectID = void 0;
/* eslint-disable multiline-comment-style */
const CommonOperations = __importStar(require("./CommonOperations"));
const State = __importStar(require("./State"));
const Logging = __importStar(require("./Logging"));
const Common = __importStar(require("./Common"));
const R = __importStar(require("runtypes"));
//////////////////////////////////////////////////////////////////////////////
/// Projects are a FS collection that as of Lhotse 4.0 only contain one    ///
/// document "1" (defaultProject), which contains a "name" field which     ///
/// is currently only used to create/identify packages. Projects are       ///
/// serialized to git in order to generate a namespace for a given package ///
//////////////////////////////////////////////////////////////////////////////
const LogProjectOperations = Logging.GetLogger(Logging.LoggingArea.Projects);
exports.defaultProjectID = "1";
//runtype that describes the query parameters for secret calls
exports.RProjectType = R.Record({
    name: R.String,
});
// cb passed into useListenFirebase
const listenProjectChangeCb = (change, dispatch) => {
    const { doc, type } = change;
    if (type === "clear") {
        dispatch({ type: State.getClearAllEntityActionString(CommonOperations.DocRefs.projects) });
    }
    else if (type === "added") {
        dispatch(State.getAddProjectAction(doc.id, doc.data()));
    }
    else if (type === "modified") {
        dispatch(State.getUpdateProjectAction(doc.id, doc.data()));
    }
    else if (type === "removed") {
        dispatch(State.getDeleteProjectAction(doc.id));
    }
    else {
        const message = `Unknown update type ${type}`;
        LogProjectOperations.error(message);
    }
};
exports.listenProjectChangeCb = listenProjectChangeCb;
const GetQualifiedProjectNameFromString = (invalidCharacters, projectName) => {
    const prunedOrgName = Common.RemoveInvalidCharactersFromString(invalidCharacters, projectName);
    return prunedOrgName.toUpperCase();
};
exports.GetQualifiedProjectNameFromString = GetQualifiedProjectNameFromString;
const GetDefaultProjectName = (projects) => {
    return Common.getValueSafe(projects, [exports.defaultProjectID, "name"], "");
};
exports.GetDefaultProjectName = GetDefaultProjectName;
const ValidateProjectAsRunType = (project) => {
    const validationResult = exports.RProjectType.validate(project);
    if (validationResult.success) {
        return { success: true, errorString: null };
    }
    else {
        return { success: false, errorString: JSON.stringify(validationResult.details, null, 2) };
    }
};
exports.ValidateProjectAsRunType = ValidateProjectAsRunType;
//# sourceMappingURL=ProjectOperations.js.map