"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DataResultHasFailedStatus = exports.GetSecureCopyForLog = exports.PlayTaskThunk = exports.GetQueryResultFromTemplateRunResult = exports.GetEmptyQueryResult = exports.RunRenderWorkspaceEditSQL = exports.GetAvailableSchemaNames = exports.GetAvailableDatabaseNames = exports.RunShowSchemas = exports.RunShowDatabases = exports.RunRenderSQL = exports.RunGetRows = exports.RunRefresh = exports.RunStepIndex = exports.RunUpdateStep = exports.RunRenderAndValidate = exports.RunStepPreview = exports.RunProvisionTable = exports.RunDeriveMappingFromCustomSQLResult = exports.RunSQLWithoutScheduler = exports.RunSQLWithToken = exports.RunSQL = exports.RunStep = exports.CreateBaseIQueryResult = exports.RunDefaultAxiosRequest = exports.RunPost = exports.RunTemplate = exports.RunShowObject = exports.RunShowViews = exports.RunShowTables = exports.RunDescObject = exports.RunDescView = exports.RunDescTable = exports.DeployLogWithPrefix = exports.CleanIUserContextForLogging = exports.runProfiles = exports.GetTemplateStringFromStepDetails = exports.OperationTypes = exports.GetBaseMetadata = exports.RunSQLStepFull = exports.GetQueryResultName = exports.CheckIsCustom = exports.GetDataFromRunResults = exports.RunAnyTemplateWithScheduler = exports.QueryResultSequence = exports.RunResults = exports.QueryResultNamePrefix = exports.RunResultsRunState = void 0;
const Runner = __importStar(require("./Runner"));
const Templates = __importStar(require("./Templates"));
const State = __importStar(require("./State"));
const Snowflake = __importStar(require("./Snowflake"));
const Steps = __importStar(require("./Steps"));
const Common = __importStar(require("./Common"));
const Workspaces = __importStar(require("./Workspaces"));
const Immer = __importStar(require("immer"));
const GraphOperations = __importStar(require("./GraphOperations"));
const StepOperations = __importStar(require("./StepOperations"));
const MacroOperations = __importStar(require("./MacroOperations"));
const Logging = __importStar(require("./Logging"));
const YAML = __importStar(require("yaml"));
const StepTypes = __importStar(require("./StepTypes"));
const Firebase = __importStar(require("./Firebase"));
const MetadataOperations = __importStar(require("./MetadataOperations"));
const Axios = __importStar(require("axios"));
const RunStepHelpers = __importStar(require("./RunStepHelpers"));
const CommonOperations = __importStar(require("./CommonOperations"));
const ErrorOperations = __importStar(require("./ErrorOperations"));
const SchedulerOperations = __importStar(require("./SchedulerOperations"));
const SQLExecutorCreators = __importStar(require("./SQLExecutorCreators"));
if (!Common.isCommandLine()) {
    window.YAML = YAML;
}
const StepTypeLogger = Logging.GetLogger(Logging.LoggingArea.StepTypes);
const RunnerLogger = Logging.GetLogger(Logging.LoggingArea.Runner);
var RunResultsRunState;
(function (RunResultsRunState) {
    RunResultsRunState["queued"] = "queued";
    RunResultsRunState["running"] = "running";
    RunResultsRunState["complete"] = "complete";
    RunResultsRunState["error"] = "error";
    RunResultsRunState["waiting"] = "waiting";
})(RunResultsRunState = exports.RunResultsRunState || (exports.RunResultsRunState = {}));
var QueryResultNamePrefix;
(function (QueryResultNamePrefix) {
    QueryResultNamePrefix["create"] = "Create";
    QueryResultNamePrefix["load"] = "Load";
    QueryResultNamePrefix["validate"] = "Validate";
    QueryResultNamePrefix["update"] = "Update";
    QueryResultNamePrefix["deploy"] = "Deploy";
    QueryResultNamePrefix["run"] = "Run";
    QueryResultNamePrefix["empty"] = "";
})(QueryResultNamePrefix = exports.QueryResultNamePrefix || (exports.QueryResultNamePrefix = {}));
class RunResults {
    constructor(dataResult, queryResultSequence) {
        this.dataResult = dataResult;
        this.runState = RunResultsRunState.waiting;
        this.stepRunState = Runner.EStepRunState.none;
    }
}
exports.RunResults = RunResults;
class QueryResultSequence {
    constructor(name, queryResults, isRunning) {
        this.name = name;
        this.queryResults = queryResults;
        this.isRunning = false;
    }
}
exports.QueryResultSequence = QueryResultSequence;
const RunAnyTemplateWithScheduler = (templateParams, token) => {
    return new Promise((resolve, reject) => {
        let result;
        if (token)
            result = (0, exports.RunSQLWithToken)(templateParams, token);
        else
            result = (0, exports.RunSQL)(templateParams);
        const startTime = new Date();
        const context = {};
        if (!!templateParams.userContext) {
            context.orgID = templateParams.userContext.orgID;
            context.envID = templateParams.userContext.environmentID;
        }
        return result.then((result) => {
            RunnerLogger.infoContext(context, "got success on run", result);
            const queryResult = (0, exports.GetQueryResultFromTemplateRunResult)(result, startTime);
            resolve(queryResult);
        }).catch((error) => {
            RunnerLogger.errorContext(context, "Got failure on template execution", error);
            reject(error);
        });
    });
};
exports.RunAnyTemplateWithScheduler = RunAnyTemplateWithScheduler;
// Helpers
const GetDataFromRunResults = (runResults, index) => {
    const hasDataResult = !!Object.keys(runResults.dataResult).length;
    const pathToGet = hasDataResult ? ["dataResult"] : ["queryResultSequence", "queryResults", index];
    return Common.getValueSafe(runResults, pathToGet, {});
};
exports.GetDataFromRunResults = GetDataFromRunResults;
const CheckIsCustom = (operation) => {
    return operation.sqlType !== Runner.ESQLType.source && operation.metadata.sourceMapping.some(source => { var _a; return (_a = source.customSQL) === null || _a === void 0 ? void 0 : _a.customSQL.length; });
};
exports.CheckIsCustom = CheckIsCustom;
/**
 * the UNKNOWN here should never actually be used, as it is a sign that either QueryResultNamePrefix.empty was passed
 *    in, or an invalid prefix type was passed in. If UNKNOWN is seen here in the app it means something has gone wrong
 */
const GetQueryResultName = (c, prefix, stepName) => {
    return c.context.isPreview ?
        `${QueryResultNamePrefix.validate} ${prefix || "UNKNOWN"} - ${stepName}` :
        `${prefix || "UNKNOWN"} - ${stepName}`;
};
exports.GetQueryResultName = GetQueryResultName;
const RunSQLStepFull = (stepCounter, executor, dispatch, getState) => {
    return new Promise((resolve, reject) => {
        const currentState = getState();
        const runFinalize = () => {
            return executor.finalize().catch(error => {
                var _a;
                RunnerLogger.emergContext({ orgID: (_a = currentState.auth) === null || _a === void 0 ? void 0 : _a.fbTeamID }, "Error in runFinalize (RunSQLStepFull):", error);
                reject();
            });
        };
        const finishSQLStep = () => {
            dispatch(State.getUpdateQueryResultSequenceIsRunningAction(stepCounter, false));
            dispatch(State.getUpdateRunResultRunStatusAction(stepCounter, RunResultsRunState.complete));
            runFinalize().then(() => {
                resolve(Runner.getMostRecentRunResult(getState().runResults[stepCounter]).queryResultSequence);
            });
        };
        const HandleTemplateRenderingError = (error) => {
            const formattedError = Runner.HandleError(error, new Date());
            if (ErrorOperations.HasTemplateRenderingError(error) || ErrorOperations.HasMissingAttributesError(error)) {
                dispatch(State.getUpdateQueryResultSequenceIndexAction(stepCounter, Object.assign(Object.assign({}, formattedError), { name: runResultName }), 0));
            }
        };
        const handleSQLStepError = (error) => {
            // for sequence level error handling
            HandleTemplateRenderingError(error);
            dispatch(State.changeUIState("sqlTabActiveIndex", "SQL"));
            dispatch(State.getUpdateRunResultRunStatusAction(stepCounter, RunResultsRunState.error));
            dispatch(State.getUpdateQueryResultSequenceIsRunningAction(stepCounter, false));
            runFinalize().then(() => {
                reject(error);
            });
        };
        if (StepOperations.checkIsStepRunning(currentState.runResults, stepCounter)) {
            reject(new Error("Steps are aleady running"));
        }
        Common.assert(Logging.LoggingArea.Runner, !!currentState.steps[stepCounter], `Step ${stepCounter} doesn't exist`);
        const runResultName = currentState.steps[stepCounter].operation.name;
        const newRunResult = {
            dataResult: {},
            runState: RunResultsRunState.waiting,
            stepRunState: Runner.EStepRunState.none,
            queryResultSequence: {
                name: runResultName,
                queryResults: [],
                isRunning: true
            }
        };
        dispatch(State.getUpdateRunResultsHistoryAction(stepCounter, newRunResult));
        dispatch(State.getUpdateQueryResultSequenceIsRunningAction(stepCounter, true));
        dispatch(State.getUpdateRunResultRunStatusAction(stepCounter, RunResultsRunState.running));
        executor.initialize().then(() => {
            executor.getStepExecutions()
                .then((stepExecutions) => {
                let currentIndex = 0;
                const executeStep = () => {
                    if (stepExecutions.length === currentIndex) {
                        finishSQLStep();
                        return;
                    }
                    return stepExecutions[currentIndex].execute(dispatch).then(queryResult => {
                        // asserting that query result is either void (valid) or it has sql
                        Common.assert(Logging.LoggingArea.Runner, !queryResult || queryResult.hasOwnProperty("sql"), "query Result checked in run sql stepfull has sql ");
                        currentIndex++;
                        return executeStep();
                    }).catch(e => {
                        return handleSQLStepError(e);
                    });
                };
                executeStep();
            })
                .catch(error => {
                // add log
                return handleSQLStepError(error);
            });
        }).catch(error => {
            return handleSQLStepError(error);
        });
    });
};
exports.RunSQLStepFull = RunSQLStepFull;
const GetBaseMetadata = (stepData, physicalLocations) => {
    return {
        destinationName: stepData.operation.locationName,
        operationName: StepOperations.GetStepName(stepData),
        mapping: physicalLocations
    };
};
exports.GetBaseMetadata = GetBaseMetadata;
/**
 * The OperationTypes and runProfiles, as well as the defaultOperationType in Runner.ts, are meant to eventually
 *    replace the template types.
 * Currently the runProfiles are only used to generate template strings for use in runSQLStepFull
 *    however they were constructed this way to be expandable in the future should we need new runProfiles or new
 *    fields for existing runProfiles
 */
var OperationTypes;
(function (OperationTypes) {
    OperationTypes["nodeMetadataSpec"] = "Node Metadata Spec";
    OperationTypes["create"] = "create";
    OperationTypes["run"] = "run";
    OperationTypes["select"] = "select";
})(OperationTypes = exports.OperationTypes || (exports.OperationTypes = {}));
const GetTemplateStringFromStepDetails = (stepTypeData, sqlType, operationType) => {
    console.assert(operationType === OperationTypes.create || operationType === OperationTypes.run);
    const templates = StepTypes.getStepTypeTemplatesSafe(stepTypeData, sqlType);
    Common.assert(Logging.LoggingArea.StepTypes, !!templates, `error with step type, ${sqlType}`);
    return templates[operationType].templateString;
};
exports.GetTemplateStringFromStepDetails = GetTemplateStringFromStepDetails;
exports.runProfiles = {
    renderAndValidateWithSelect: {
        operationType: OperationTypes.select,
        renderAndValidate: true
    },
    renderAndValidateWithCreate: {
        operationType: OperationTypes.create,
        renderAndValidate: true
    },
    renderAndValidateWithRun: {
        operationType: OperationTypes.run,
        renderAndValidate: true
    },
    create: {
        operationType: OperationTypes.create,
        renderAndValidate: false
    },
    run: {
        operationType: OperationTypes.run,
        renderAndValidate: false
    }
};
/**
 * These ref types are the keys inside of the dictionary TTemplateRunSuccess.ref_usage, see TRefUsage below
 * The value of each of these keys is IAPIRefLink[] returned from the json templating api
 *
 * These ref types map accordingly with MacroOperations.RefMacros, which are utilized in areas such as the join editor
 * in its response, json templating api parses out the different types of refs (ERefTypes) and fills the appropriate array
 * A ref type with the prefix `invalid_x` will have a value when the ref x has an invalid storage location
 * (note: as of 2/23, we do not detect invalid node name before sending off to json template api)
 */
var ERefTypes;
(function (ERefTypes) {
    ERefTypes["invalid_ref"] = "invalid_ref";
    ERefTypes["invalid_ref_link"] = "invalid_ref_link";
    ERefTypes["invalid_ref_no_link"] = "invalid_ref_no_link";
    ERefTypes["ref"] = "ref";
    ERefTypes["ref_link"] = "ref_link";
    ERefTypes["ref_no_link"] = "ref_no_link";
})(ERefTypes || (ERefTypes = {}));
const CleanIUserContextForLogging = (userContext) => {
    return {
        user: userContext.user,
        accountName: userContext.accountName,
        database: userContext.database,
        destinationSchema: userContext.destinationSchema,
        warehouse: userContext.warehouse,
        role: userContext.role,
        orgID: userContext.orgID,
        environmentID: userContext.environmentID,
        authenticator: userContext.authenticator
    };
};
exports.CleanIUserContextForLogging = CleanIUserContextForLogging;
const DeployLogWithPrefix = (deployContext) => {
    const DeployLogger = Logging.GetLogger(Logging.LoggingArea.Deploy);
    const logContext = deployContext.logContext;
    const info = (...logStatements) => {
        return DeployLogger.infoContext(logContext, `runCounter: ${deployContext.runCounter};`, ...logStatements);
    };
    const emerg = (...logStatements) => {
        return DeployLogger.emergContext(logContext, `runCounter: ${deployContext.runCounter};`, ...logStatements);
    };
    const alert = (...logStatements) => {
        return DeployLogger.alertContext(logContext, `runCounter: ${deployContext.runCounter};`, ...logStatements);
    };
    const error = (...logStatements) => {
        return DeployLogger.errorContext(logContext, `runCounter: ${deployContext.runCounter};`, ...logStatements);
    };
    const debug = (...logStatements) => {
        return DeployLogger.debugContext(logContext, `runCounter: ${deployContext.runCounter};`, ...logStatements);
    };
    return { info, emerg, alert, error, debug };
};
exports.DeployLogWithPrefix = DeployLogWithPrefix;
const RunDescTable = (locations, stepData, UserContext, macros) => {
    return (0, exports.RunDescObject)(locations, stepData, UserContext, macros, Templates.GenericTemplateTypes.describe);
};
exports.RunDescTable = RunDescTable;
const RunDescView = (locations, stepData, UserContext, macros) => {
    return (0, exports.RunDescObject)(locations, stepData, UserContext, macros, Templates.GenericTemplateTypes.describeView);
};
exports.RunDescView = RunDescView;
const RunDescObject = (locations, stepData, UserContext, macros, genericTemplateType) => {
    if (!stepData) {
        console.assert(false);
        throw new Error("step data did not exist");
    }
    const { locationID, locationName } = stepData;
    const locationMapping = locations[locationID];
    const table = stepData.name;
    const userContext = Object.assign({}, UserContext);
    /*
     * userContext.database = locationMapping.database
     * userContext.destinationSchema = locationMapping.schema
     */
    const describeTableTemplate = Templates.getGenericTemplate(genericTemplateType);
    const parsedYamlResult = Templates.getParsedSQLStagesFromString(describeTableTemplate);
    if (!!parsedYamlResult.error) {
        return Promise.reject(parsedYamlResult.error);
    }
    // in the case of describe, we're assuming for now that there's only one sql stage involved
    console.assert(parsedYamlResult.parsedSQLStages.length === 1);
    const templateParams = {
        userContext: userContext,
        templateString: parsedYamlResult.parsedSQLStages[0].code,
        nodeMetadata: { operationName: table, destinationName: locationName, mapping: { [locationID]: locationMapping } },
        macrosString: MacroOperations.getAllMacros(macros)
    };
    return (0, exports.RunTemplate)(templateParams, null);
};
exports.RunDescObject = RunDescObject;
const RunShowTables = (locations, stepData, UserContext, macros) => {
    return (0, exports.RunShowObject)(locations, stepData, UserContext, macros, Templates.GenericTemplateTypes.showTables);
};
exports.RunShowTables = RunShowTables;
const RunShowViews = (locations, stepData, UserContext, macros) => {
    return (0, exports.RunShowObject)(locations, stepData, UserContext, macros, Templates.GenericTemplateTypes.showViews);
};
exports.RunShowViews = RunShowViews;
const RunShowObject = (locations, stepData, UserContext, macros, genericTemplateType) => {
    if (!stepData) {
        console.assert(false);
        throw new Error("step data did not exist");
    }
    const { locationID, locationName } = stepData;
    const locationMapping = locations[locationID];
    const table = stepData.name;
    const userContext = Object.assign({}, UserContext);
    /*
     * userContext.database = locationMapping.database
     * userContext.destinationSchema = locationMapping.schema
     */
    const locationString = `${(locationMapping === null || locationMapping === void 0 ? void 0 : locationMapping.database) || ""}.${(locationMapping === null || locationMapping === void 0 ? void 0 : locationMapping.schema) || ""}`;
    const showTablesTemplate = Templates.getGenericTemplate(genericTemplateType);
    const parsedYamlResult = Templates.getParsedSQLStagesFromString(showTablesTemplate);
    if (!!parsedYamlResult.error) {
        return Promise.reject(parsedYamlResult.error);
    }
    // in the case of show tables, we're assuming for now that there's only one sql stage involved
    console.assert(parsedYamlResult.parsedSQLStages.length === 1);
    const templateParams = {
        userContext: userContext,
        templateString: parsedYamlResult.parsedSQLStages[0].code,
        nodeMetadata: { operationName: table, destinationName: locationString, mapping: { [locationID]: locationMapping } },
        macrosString: MacroOperations.getAllMacros(macros)
    };
    return (0, exports.RunTemplate)(templateParams, null);
};
exports.RunShowObject = RunShowObject;
//renders sql and then executes
const RunTemplate = (templateParams, token) => {
    var _a, _b, _c;
    const log = `Front-end/runTemplate: ${templateParams.templateString}`;
    const logContext = {
        orgID: (_a = templateParams.userContext) === null || _a === void 0 ? void 0 : _a.orgID,
        userID: (_b = templateParams.userContext) === null || _b === void 0 ? void 0 : _b.user,
        envID: (_c = templateParams.userContext) === null || _c === void 0 ? void 0 : _c.environmentID
    };
    return Common.timedLoggerPromise(log, logContext, Logging.LoggingArea.RunnerBackend, new Promise((resolve, reject) => {
        Runner.RunPost("/api/json_template/", Object.assign(Object.assign({}, templateParams), { renderTemplateOnly: true }))
            .then((result) => {
            if (result.data.error) {
                reject(!!result.data.error);
            }
            else {
                resolve(result.data);
            }
        }).catch(reject); //on connection errors with runpost, rethrow
    })).then((templateRunSuccess) => {
        if (templateParams.renderTemplateOnly) {
            return templateRunSuccess;
        }
        else {
            console.log("running sql", Object.assign(Object.assign({}, templateParams), { userContext: null }));
            const templateParamsCopy = Object.assign({}, templateParams); //JS didn't like assigning to an object passed in as a param, need copy here
            templateParamsCopy.templateString = templateRunSuccess.runDetails.compiledSQL;
            let promise;
            if (token) {
                promise = () => (0, exports.RunSQLWithToken)(templateParamsCopy, token);
            }
            else {
                promise = () => (0, exports.RunSQL)(templateParamsCopy);
            }
            return promise(templateParamsCopy)
                .then((runSQLResult) => {
                RunnerLogger.debug("got runsql result", runSQLResult);
                return Object.assign(Object.assign({}, templateRunSuccess), { data: runSQLResult.data });
            })
                .catch((err) => {
                RunnerLogger.errorContext(logContext, "Error running template", err);
                throw err;
            });
        }
    });
};
exports.RunTemplate = RunTemplate;
//can throw a RunError
const RunPost = (url, data) => {
    return Axios.default.post("" + url, data)
        .catch((err) => {
        var _a, _b, _c, _d, _e;
        const moreErrData = (_b = (_a = err.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.error;
        const compiledSQL = (_e = (_d = (_c = err.response) === null || _c === void 0 ? void 0 : _c.data) === null || _d === void 0 ? void 0 : _d.runDetails) === null || _e === void 0 ? void 0 : _e.compiledSQL;
        console.log("Run Post additional error detail: ", moreErrData);
        let formattedRealError = {
            error: Object.assign({}, moreErrData)
        };
        if (compiledSQL) {
            formattedRealError["runDetails"] = { compiledSQL };
        }
        throw formattedRealError;
    });
};
exports.RunPost = RunPost;
/**
 * config must include the url and request method
 */
const RunDefaultAxiosRequest = (config) => {
    return Axios.default(config)
        .catch((err) => {
        throw err;
    });
};
exports.RunDefaultAxiosRequest = RunDefaultAxiosRequest;
const CreateBaseIQueryResult = () => {
    return {
        sql: "",
        success: true,
        rows: [],
        fields: [],
        error: null,
        startTime: new Date(),
        endTime: new Date(),
        name: "dummy",
        isRunning: false,
        exportedRefs: [],
        invalidExportedRefs: [],
        queryResultSequence: null,
        status: Snowflake.ESnowflakeDataReturnStatus.success,
        type: Templates.EQueryResultType.sql
    };
};
exports.CreateBaseIQueryResult = CreateBaseIQueryResult;
const RunStep = (stepCounter, executor) => {
    return (dispatch, getState) => {
        return new Promise((resolve, reject) => {
            (0, exports.RunSQLStepFull)(stepCounter, executor, dispatch, getState).then((result) => {
                RunnerLogger.info("in resolving result");
                resolve(result);
            }).catch((result) => {
                reject(result);
            });
        });
    };
};
exports.RunStep = RunStep;
//only executes sql 
const RunSQL = (templateParams) => {
    var _a;
    return (_a = Firebase.getFirebaseAuth().currentUser) === null || _a === void 0 ? void 0 : _a.getIdToken(false).then((idToken) => {
        return (0, exports.RunSQLWithToken)(templateParams, idToken);
    });
};
exports.RunSQL = RunSQL;
//only executes sql 
const RunSQLWithToken = (templateParams, token) => {
    const log = `Front-end/runSQL: ${templateParams.templateString}`;
    const logContext = {
        orgID: templateParams.userContext.orgID,
        userID: templateParams.userContext.user,
        envID: templateParams.userContext.environmentID
    };
    return Common.timedLoggerPromise(log, logContext, Logging.LoggingArea.RunnerBackend, new Promise((resolve, reject) => {
        return (0, exports.RunPost)(Common.START_RUN_URL + `/scheduler/run?id_token=${token}`, templateParams)
            .then((result) => {
            if (!!result.data.error) {
                reject(result.data.error);
            }
            else {
                resolve(result.data);
            }
        }).catch((err) => {
            reject(err);
        }); //on connection errors with runpost, rethrow
    }));
};
exports.RunSQLWithToken = RunSQLWithToken;
const RunSQLWithoutScheduler = (templateParams, teamDetailsAndFirebase, connectionCache) => {
    const log = `Front-end/runSQL: ${templateParams.templateString}`;
    const logContext = {
        orgID: templateParams.userContext.orgID,
        userID: templateParams.userContext.user,
        envID: templateParams.userContext.environmentID
    };
    return Common.timedLoggerPromise(log, logContext, Logging.LoggingArea.RunnerBackend, new Promise((resolve, reject) => {
        return SchedulerOperations.RunSQLWithoutScheduler(templateParams, connectionCache, teamDetailsAndFirebase)
            .then((result) => {
            resolve(result);
        }).catch(reject);
    }));
};
exports.RunSQLWithoutScheduler = RunSQLWithoutScheduler;
const RunDeriveMappingFromCustomSQLResult = (stepCounter, teamID, macros, errorAlert) => {
    return (dispatch, getState) => {
        const state = getState();
        const workspaceID = Workspaces.getCurrentWorkspace(state);
        const locations = Workspaces.getCurrentWorkspaceMappings(state);
        const userContext = CommonOperations.GetUserContextByUserAuthType(state, workspaceID, null);
        const stepData = state.steps[stepCounter].operation;
        const startTime = new Date();
        if (stepData.materializationType !== "view") {
            const errorMessage = "Derive Mapping currently only supported for Materialization Type: View";
            errorAlert(errorMessage);
            return new Promise((resolve, reject) => {
                reject(errorMessage);
            });
        }
        // run view's custom sql
        return dispatch((0, exports.RunProvisionTable)(stepCounter, false))
            .then(() => {
            // run describe on the result of the custom sql
            return (0, exports.RunDescView)(locations, stepData, userContext, macros);
        })
            .then(templateRunResult => {
            // format the describe result into column metadata
            const dataResult = (0, exports.GetQueryResultFromTemplateRunResult)(templateRunResult, startTime);
            return MetadataOperations.GetColumnMetadataFromDescTableResult(dataResult, teamID, stepCounter);
        })
            .catch(err => {
            console.error(err);
            throw new Error("Error deriving mapping");
        });
    };
};
exports.RunDeriveMappingFromCustomSQLResult = RunDeriveMappingFromCustomSQLResult;
const GetTemplateStringForStepID = (stepID, allSteps, parsedStepTypes, operationType) => {
    const sqlType = allSteps[stepID].operation.sqlType;
    const templates = StepTypes.getStepTypeTemplatesSafe(parsedStepTypes, sqlType);
    return templates[operationType].templateString;
};
const RunProvisionTable = (stepCounter, isPreview) => {
    return (dispatch, getState) => {
        const currentState = getState();
        const templateString = GetTemplateStringForStepID(stepCounter, currentState.steps, currentState.stepTypesParsed, OperationTypes.create);
        const runtimeParameters = Workspaces.GetRunTimeParametersFromWorkspacesSafe(currentState.workspaces, currentState.workspaces.currentWorkspace);
        const initializeCB = () => {
            dispatch(State.getUpdateStepRunStateAction(stepCounter, Runner.EStepRunState.created));
            return Promise.resolve();
        };
        const getMetadataForRunProvisionTable = () => {
            return MetadataOperations.GetHydratedMetadataWithNodeMetadata(stepCounter, currentState, OperationTypes.create, null, null, runtimeParameters);
        };
        const hydratedMetadata = getMetadataForRunProvisionTable();
        const RunSQL = SQLExecutorCreators.CreateRunSQLWithScheduler(null);
        const userContext = CommonOperations.GetUserContextByUserAuthType(currentState, Workspaces.getCurrentWorkspaceID(currentState), null);
        const executor = RunStepHelpers.GetRunStepExecutor(templateString, currentState, null, stepCounter, initializeCB, false, RunSQL, isPreview, hydratedMetadata, userContext, runtimeParameters);
        return (0, exports.RunSQLStepFull)(stepCounter, executor, dispatch, getState);
    };
};
exports.RunProvisionTable = RunProvisionTable;
const RunStepPreview = (stepCounter, specifiedIndex) => {
    return (dispatch, getState) => {
        const currentState = getState();
        const templateString = Templates.getPreviewString(Templates.PlatformType.snowflake);
        const runtimeParameters = Workspaces.GetRunTimeParametersFromWorkspacesSafe(currentState.workspaces, currentState.workspaces.currentWorkspace);
        const initializeCB = () => {
            dispatch(State.getUpdateStepRunStateAction(stepCounter, Runner.EStepRunState.validated));
            return Promise.resolve();
        };
        const getMetadataForRunStepPreview = () => {
            return MetadataOperations.GetHydratedMetadataWithNodeMetadata(stepCounter, getState(), OperationTypes.select, specifiedIndex, null, runtimeParameters);
        };
        const hydratedMetadata = getMetadataForRunStepPreview();
        const RunSQL = SQLExecutorCreators.CreateRunSQLWithScheduler(null);
        const userContext = CommonOperations.GetUserContextByUserAuthType(currentState, Workspaces.getCurrentWorkspaceID(currentState), null);
        const executor = RunStepHelpers.GetRunStepExecutor(templateString, getState(), specifiedIndex, stepCounter, initializeCB, false, RunSQL, true, hydratedMetadata, userContext, runtimeParameters);
        return (0, exports.RunSQLStepFull)(stepCounter, executor, dispatch, getState);
    };
};
exports.RunStepPreview = RunStepPreview;
const RunRenderAndValidate = (stepCounter, operationType, templateEditorOverride) => {
    return (dispatch, getState) => {
        const currentState = getState();
        const nodeType = currentState.templateEditor.nodeType;
        const templates = StepTypes.getStepTypeTemplatesSafe(currentState.stepTypesParsed, nodeType);
        const templateString = templates[operationType].templateString;
        const runtimeParameters = Workspaces.GetRunTimeParametersFromWorkspacesSafe(currentState.workspaces, currentState.workspaces.currentWorkspace);
        const initializeCB = () => {
            return Promise.resolve();
        };
        const getMetadataForRunRenderAndValidate = () => {
            return MetadataOperations.GetHydratedMetadataWithNodeMetadata(stepCounter, currentState, operationType, null, templateEditorOverride, runtimeParameters);
        };
        const hydratedMetadata = getMetadataForRunRenderAndValidate();
        const RunSQL = SQLExecutorCreators.CreateRunSQLWithScheduler(null);
        const userContext = CommonOperations.GetUserContextByUserAuthType(currentState, Workspaces.getCurrentWorkspaceID(currentState), null);
        const executor = RunStepHelpers.GetRunStepExecutor(templateString, currentState, null, stepCounter, initializeCB, false, RunSQL, true, hydratedMetadata, userContext, runtimeParameters);
        return (0, exports.RunSQLStepFull)(stepCounter, executor, dispatch, getState);
    };
};
exports.RunRenderAndValidate = RunRenderAndValidate;
const RunUpdateStep = (stepCounter, isPreview) => {
    return (dispatch, getState) => {
        const currentState = getState();
        const templateString = GetTemplateStringForStepID(stepCounter, currentState.steps, currentState.stepTypesParsed, OperationTypes.run);
        const runtimeParameters = Workspaces.GetRunTimeParametersFromWorkspacesSafe(currentState.workspaces, currentState.workspaces.currentWorkspace);
        const initializeCB = () => {
            dispatch(State.getUpdateStepRunStateAction(stepCounter, Runner.EStepRunState.loaded));
            return Promise.resolve();
        };
        const getMetadataForRunUpdateStep = () => {
            return MetadataOperations.GetHydratedMetadataWithNodeMetadata(stepCounter, currentState, OperationTypes.run, null, null, runtimeParameters);
        };
        const hydratedMetadata = getMetadataForRunUpdateStep();
        const RunSQL = SQLExecutorCreators.CreateRunSQLWithScheduler(null);
        const userContext = CommonOperations.GetUserContextByUserAuthType(currentState, Workspaces.getCurrentWorkspaceID(currentState), null);
        const executor = RunStepHelpers.GetRunStepExecutor(templateString, currentState, null, stepCounter, initializeCB, true, RunSQL, isPreview, hydratedMetadata, userContext, runtimeParameters);
        return (0, exports.RunSQLStepFull)(stepCounter, executor, dispatch, getState)
            .then(queryResult => queryResult);
    };
};
exports.RunUpdateStep = RunUpdateStep;
const RunStepIndex = (stepCounter, index) => {
    return (dispatch, getState) => {
        const currentState = getState();
        const templateString = GetTemplateStringForStepID(stepCounter, currentState.steps, currentState.stepTypesParsed, OperationTypes.run);
        const runtimeParameters = Workspaces.GetRunTimeParametersFromWorkspacesSafe(currentState.workspaces, currentState.workspaces.currentWorkspace);
        const initializeCB = () => {
            dispatch(State.getUpdateStepRunStateAction(stepCounter, Runner.EStepRunState.loaded));
            return Promise.resolve();
        };
        const getMetadataForRunStepIndex = () => {
            return MetadataOperations.GetHydratedMetadataWithNodeMetadata(stepCounter, currentState, OperationTypes.run, index, null, runtimeParameters);
        };
        //wrap RunAnyTemplate to create a RunStepHelpers.SQLPlatformExecutor
        const hydratedMetadata = getMetadataForRunStepIndex();
        const RunSQL = SQLExecutorCreators.CreateRunSQLWithScheduler(null);
        const userContext = CommonOperations.GetUserContextByUserAuthType(currentState, Workspaces.getCurrentWorkspaceID(currentState), null);
        const executor = RunStepHelpers.GetRunStepExecutor(templateString, currentState, index, stepCounter, initializeCB, true, RunSQL, false, hydratedMetadata, userContext, runtimeParameters);
        return (0, exports.RunSQLStepFull)(stepCounter, executor, dispatch, getState);
    };
};
exports.RunStepIndex = RunStepIndex;
const RunRefresh = (stepCounter) => {
    return (dispatch, getState) => {
        const startTime = new Date();
        const c = {
            dispatch, getState, context: {}, templateType: Runner.RunTemplateTypes.default, onRunResult: null
        };
        dispatch(State.newDataResultAction(stepCounter, { isRunning: true }));
        return dispatch((0, exports.RunGetRows)(stepCounter, null)).then((queryResult) => {
            dispatch(State.newDataResultAction(stepCounter, queryResult));
            return queryResult;
        }).catch(error => {
            const resultError = Runner.HandleError(error, startTime);
            c.dispatch(State.newDataResultAction(stepCounter, resultError));
        });
    };
};
exports.RunRefresh = RunRefresh;
const RunGetRows = (stepCounter, token) => {
    return (dispatch, getState) => {
        const state = getState();
        const workspaceID = Workspaces.getCurrentWorkspace(state);
        const userContext = CommonOperations.GetUserContextByUserAuthType(state, workspaceID, null);
        const c = {
            dispatch, getState, context: {}, templateType: Runner.RunTemplateTypes.getData, onRunResult: null
        };
        const stepData = getState().steps[stepCounter];
        const operation = stepData.operation;
        const locations = Workspaces.getCurrentWorkspaceMappings(getState());
        const allMacros = Object.assign(Object.assign({}, c.getState().macros), MacroOperations.refMacroDefinitions);
        let database = null, dataset = null, table = null;
        if (operation.sqlType == Runner.ESQLType.source) {
            database = operation.database;
            dataset = operation.dataset;
            table = StepOperations.GetStepName(stepData);
        }
        else {
            table = StepOperations.GetStepName(stepData);
        }
        const start = new Date();
        return Snowflake.runGetRows(userContext, operation, locations, allMacros, token).then((queryResult) => {
            dispatch(State.newDataResultAction(stepCounter, queryResult));
            return queryResult;
        });
    };
};
exports.RunGetRows = RunGetRows;
const RunRenderSQL = (templateString, userContext, nodeMetadata, macrosString) => {
    const templateParams = {
        templateString,
        userContext,
        nodeMetadata,
        macrosString,
        renderTemplateOnly: true,
        formatSQL: false,
    };
    return (0, exports.RunTemplate)(templateParams, null);
};
exports.RunRenderSQL = RunRenderSQL;
const RunShowDatabases = (currentState, workspaceID) => {
    const templateParams = {
        templateString: Templates.TemplateString.SHOW_DATABASES,
        userContext: CommonOperations.GetUserContextByUserAuthType(currentState, workspaceID, null),
        macrosString: "",
        formatSQL: false,
        nodeMetadata: {}
    };
    return (0, exports.RunSQL)(templateParams);
};
exports.RunShowDatabases = RunShowDatabases;
const GetFullShowSchemasString = (databaseName) => {
    return `${Templates.TemplateString.SHOW_SCHEMAS} IN ${databaseName}`;
};
const RunShowSchemas = (currentState, databaseName, environmentID) => {
    const templateParams = {
        templateString: GetFullShowSchemasString(databaseName),
        userContext: CommonOperations.GetUserContextByUserAuthType(currentState, environmentID, null),
        macrosString: "",
        formatSQL: false,
        nodeMetadata: {}
    };
    return (0, exports.RunSQL)(templateParams);
};
exports.RunShowSchemas = RunShowSchemas;
const GetAvailableDatabaseNames = (currentState, workspaceID) => {
    return (0, exports.RunShowDatabases)(currentState, workspaceID).then(result => {
        return result.data.rows.map(row => {
            return row.name;
        });
    }).catch(error => {
        throw error;
    });
};
exports.GetAvailableDatabaseNames = GetAvailableDatabaseNames;
const GetAvailableSchemaNames = (currentState, databaseName, environmentID) => {
    return (0, exports.RunShowSchemas)(currentState, databaseName, environmentID).then(result => {
        return result.data.rows.map(row => {
            return row.name;
        });
    }).catch(error => {
        throw error;
    });
};
exports.GetAvailableSchemaNames = GetAvailableSchemaNames;
const RunRenderWorkspaceEditSQL = (editsCollection, userContext, macros) => {
    const editsCollectionRendered = {};
    const renderPromises = [];
    //do my async stuff
    Steps.forAllSteps(editsCollection, (stepCounter, stepData) => {
        //generate promises
        const templateParams = {
            userContext: userContext,
            nodeMetadata: stepData.nodeMetadata,
            macrosString: MacroOperations.getAllMacros(macros),
            renderTemplateOnly: true,
            templateString: ""
        };
        //setup editsCollectionRendered for the current step
        editsCollectionRendered[stepCounter] = {};
        stepData.workspaceEdits.forEach((edit) => {
            //push extra info into nodeMetadata
            templateParams.templateString = edit.sql;
            templateParams.nodeMetadata = Object.assign(Object.assign({}, templateParams.nodeMetadata), edit.dataForNodeMetadata);
            renderPromises.push((0, exports.RunTemplate)(templateParams, null).then(result => {
                editsCollectionRendered[stepCounter][edit.description] = result.runDetails.compiledSQL;
            }));
        });
    });
    return Promise.all(renderPromises)
        .then(() => {
        return editsCollectionRendered;
    })
        .catch(error => {
        console.error(error);
        throw error;
    });
};
exports.RunRenderWorkspaceEditSQL = RunRenderWorkspaceEditSQL;
const GetEmptyQueryResult = (templateRunResult, startTime) => {
    return Object.assign(Object.assign({}, (0, exports.CreateBaseIQueryResult)()), { startTime, name: "", status: Common.getValueSafe(templateRunResult, ["data", "status"], Snowflake.ESnowflakeDataReturnStatus.success) });
};
exports.GetEmptyQueryResult = GetEmptyQueryResult;
const GetQueryResultFromTemplateRunResult = (templateRunResult, startTime) => {
    var _a, _b;
    const sql = templateRunResult.runDetails.compiledSQL;
    const queryResult = {
        error: null,
        fields: [],
        rows: [],
        success: true,
        sql,
        exportedRefs: [],
        invalidExportedRefs: [],
        startTime,
        endTime: new Date(),
        isRunning: false,
        name: "",
        status: Common.getValueSafe(templateRunResult, ["data", "status"], Snowflake.ESnowflakeDataReturnStatus.success),
        // default to sql here - this gets set properly in the executor, where we have access to the queryResultType of the stage
        type: Templates.EQueryResultType.sql
    };
    if (templateRunResult.data) {
        const rows = templateRunResult.data.rows.map((row) => {
            return { values: Object.values(row).map(v => v) };
        });
        const fields = templateRunResult.data.columns.map((c) => {
            let field = {};
            field.name = c.Name;
            if (c.DataType) {
                field.type = c.DataType;
            }
            return field;
        });
        queryResult.fields = fields;
        queryResult.rows = rows;
    }
    // update exportedRefs and invalidExportedRefs
    if ((_a = templateRunResult === null || templateRunResult === void 0 ? void 0 : templateRunResult.runDetails) === null || _a === void 0 ? void 0 : _a.exported_refs.length) {
        const exportedRefs = templateRunResult.runDetails.exported_refs.map(ref => {
            return {
                locationName: ref.location_name, nodeName: ref.node_name
            };
        });
        queryResult.exportedRefs = exportedRefs;
    }
    if ((_b = templateRunResult === null || templateRunResult === void 0 ? void 0 : templateRunResult.runDetails) === null || _b === void 0 ? void 0 : _b.invalid_exported_refs.length) {
        const invalidRefs = templateRunResult.runDetails.invalid_exported_refs.map(ref => {
            return {
                locationName: ref.location_name, nodeName: ref.node_name
            };
        });
        queryResult.invalidExportedRefs = invalidRefs;
    }
    return queryResult;
};
exports.GetQueryResultFromTemplateRunResult = GetQueryResultFromTemplateRunResult;
/**
 *
 * @param dataSelect
 * @param stepsSubset an optional arg to specify a subset of steps to iterate through
 * @param token
 * can either be a specified token or null - if null, the code path will use our firebase instance
 * from Shared to get the token with Shared.Firebase.getFirebaseAuth().currentUser?.refreshToken.
 * pass a token for any other instance where the Shared firebase instance may be invalid (i.e. refreshes)
 * @param specifiedExecutorFactory
 * either a function passed in that adheres to TExecutorFactory typing or null - if null, we default to
 * grabbing an instance of the RunStepExecutor which covers the majority of use cases.
 * @param operationType
 * determines which template string to pass into the executor - allows us to run creates or runs here.
 * @returns
 */
const outstandingRequestsLimit = 64;
function PlayTaskThunk(dataSelect, stepsSubset, specifiedExecutorFactory, operationType, RunSQL, isPreview, runtimeParameters) {
    return function (dispatch, getState) {
        return new Promise((resolve, reject) => {
            const currentState = getState();
            const fullLogContext = {
                logContext: {
                    orgID: currentState.auth.fbTeamID,
                    userID: currentState.auth.fbUserID,
                    envID: currentState.currentWorkspace
                },
                runCounter: "N/A"
            };
            if (StepOperations.checkAreStepsRunning(currentState.runResults, currentState.steps)) {
                reject(new Error("Steps are already running"));
                return;
            }
            let stepsInTree = Steps.getAllSteps(getState().steps);
            if (stepsSubset) {
                stepsInTree = stepsInTree.filter(s => stepsSubset.indexOf(s) != -1);
            }
            dispatch(State.getSetQueuedAction(stepsInTree));
            let numberOfRequestsOutstanding = 0; //number of outstanding steps running
            const processNext = () => {
                const currentState = getState();
                let allComplete = true;
                stepsInTree.forEach((s) => {
                    const runResult = currentState.runResults[s];
                    const currentRunResultsStatus = runResult.runState;
                    if (currentRunResultsStatus && (currentRunResultsStatus == "running" ||
                        currentRunResultsStatus == "queued")) {
                        allComplete = false;
                    }
                });
                if (allComplete) {
                    RunnerLogger.infoContext(fullLogContext.logContext, "all complete for run (PlayTaskThunk)");
                    resolve();
                    return;
                }
                const dependenciesLookup = GraphOperations.getDependenciesLookup(currentState.steps);
                stepsInTree.forEach((s) => {
                    const currentState = getState();
                    if (numberOfRequestsOutstanding > outstandingRequestsLimit)
                        return;
                    Common.assert(Logging.LoggingArea.Runner, !!currentState.steps[s], `PlayTaskThunk error: Step ${s} doesn't exist`);
                    const runResult = currentState.runResults[s];
                    const isCompleteOrRunning = runResult.runState === RunResultsRunState.running ||
                        runResult.runState === RunResultsRunState.complete ||
                        runResult.runState === RunResultsRunState.error;
                    if (isCompleteOrRunning) {
                        //it has completed or is running already
                        (0, exports.DeployLogWithPrefix)(fullLogContext).debug("completed or running already", s);
                        return;
                    }
                    let allDependenciesReady = true;
                    const dependencies = dependenciesLookup[s];
                    dependencies.forEach((dep) => {
                        if (!stepsInTree.includes(dep)) {
                            (0, exports.DeployLogWithPrefix)(fullLogContext).debug(`skipping dep: ${dep} - not in stepsInTree`);
                        }
                        else if (!(dep in currentState.runResults)) {
                            //this dependency was deleted 
                            (0, exports.DeployLogWithPrefix)(fullLogContext).error("dep doesnt exist in run results", currentState.runResults, dep);
                        }
                        else {
                            const dependencyStatus = currentState.runResults[dep].runState;
                            if (dependencyStatus == "queued" || dependencyStatus == "running" || dependencyStatus == "waiting") {
                                (0, exports.DeployLogWithPrefix)(fullLogContext).debug(`dep ${dep} status: ${dependencyStatus}`);
                                allDependenciesReady = false;
                            }
                        }
                    });
                    if (allDependenciesReady) {
                        const templateString = GetTemplateStringForStepID(s, currentState.steps, currentState.stepTypesParsed, operationType);
                        let executor;
                        if (specifiedExecutorFactory) {
                            executor = specifiedExecutorFactory(templateString, currentState, null, s, false, RunSQL, runtimeParameters);
                        }
                        else {
                            // baseline behavior how each step should run
                            const initializeCB = () => {
                                dispatch(State.getUpdateStepRunStateAction(s, Runner.EStepRunState.loaded));
                                return Promise.resolve();
                            };
                            const materializationType = Common.getValueSafe(currentState.steps, [s, "operation", "materializationType"], Templates.materializationTypes.table);
                            const shouldFetchDataOnSuccess = materializationType === Templates.materializationTypes.table && dataSelect;
                            const getMetadataForPlayTaskThunk = () => {
                                return MetadataOperations.GetHydratedMetadataWithNodeMetadata(s, currentState, operationType, null, null, runtimeParameters);
                            };
                            const hydratedMetadata = getMetadataForPlayTaskThunk();
                            const userContext = CommonOperations.GetUserContextByUserAuthType(currentState, Workspaces.getCurrentWorkspaceID(currentState), null);
                            executor = RunStepHelpers.GetRunStepExecutor(templateString, currentState, null, s, initializeCB, shouldFetchDataOnSuccess, RunSQL, isPreview, hydratedMetadata, userContext, runtimeParameters);
                        }
                        //===== RunStep =====
                        numberOfRequestsOutstanding++;
                        dispatch(Runner.RunStep(s, executor)).then((result) => {
                            (0, exports.DeployLogWithPrefix)(fullLogContext).info(`step:${s} finished running (PlayTaskThunk runStep) outstanding:${numberOfRequestsOutstanding}`);
                            processNext();
                        })
                            .catch((error) => {
                            (0, exports.DeployLogWithPrefix)(fullLogContext).error(`PlayTaskThunk runStep finished with error:`, error);
                            reject(error);
                        }).finally(() => {
                            numberOfRequestsOutstanding--;
                        });
                    }
                });
            };
            processNext();
        });
    };
}
exports.PlayTaskThunk = PlayTaskThunk;
// HELPERS
/**
 * Removes sensitive user info from any object that may be carrying it
 * @param anyObject
 * @returns the same object without user credentials
 */
const GetSecureCopyForLog = (anyObject) => {
    if (!anyObject)
        return anyObject;
    const secureState = Immer.produce(anyObject, draftState => {
        delete draftState["userConfig"];
        delete draftState["userContext"];
        delete draftState["userCredentials"];
    });
    return secureState;
};
exports.GetSecureCopyForLog = GetSecureCopyForLog;
const DataResultHasFailedStatus = (dataResult) => {
    return dataResult.status && dataResult.status !== Snowflake.ESnowflakeDataReturnStatus.success;
};
exports.DataResultHasFailedStatus = DataResultHasFailedStatus;
//# sourceMappingURL=RunnerOperations.js.map