"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteUserConnectionUsernameByEnvironment = exports.DeleteUserConnectionByEnvironment = exports.DeleteUserConnectionByEnvironmentAndConnectionType = exports.DeleteSecret = exports.AuthenticateAndDeleteSecret = exports.ValidateAndDeleteSecret = exports.GetSecret = exports.AuthenticateFBAndGetSecret = exports.GetSecretParentLatest = exports.GetSecretParent = exports.GetRootSecretParent = exports.GetSecretID = exports.ValidateSecretRequest = exports.ValidateAndGetSecret = exports.UpdateSecret = exports.AuthenticateAndUpdateSecret = exports.CreateSecretIfNotExists = exports.ValidateAndUpdateSecret = exports.GetSecretsManager = exports.RunnerBackendLogger = exports.SecretQueryParamsType = void 0;
const R = __importStar(require("runtypes"));
const secrets = __importStar(require("@google-cloud/secret-manager"));
const Common = __importStar(require("./Common"));
const Logging = __importStar(require("./Logging"));
const Firebase = __importStar(require("./Firebase"));
const SchedulerOperations = __importStar(require("./SchedulerOperations"));
const ConnectionOperations = __importStar(require("./ConnectionOperations"));
//runtype that describes the query parameters for secret calls
exports.SecretQueryParamsType = R.Record({
    id_token: R.String,
    secretID: R.String
});
/**
 * ===== SETUP =====
 */
exports.RunnerBackendLogger = Logging.GetLogger(Logging.LoggingArea.RunnerBackend);
let SecretManager;
if (Common.isCommandLine()) {
    SecretManager = new secrets.SecretManagerServiceClient();
}
const GetSecretsManager = () => {
    return SecretManager;
};
exports.GetSecretsManager = GetSecretsManager;
/**
 * ===== CREATE/UPDATE SECRETS =====
 */
const ValidateAndUpdateSecret = (req, res) => {
    const result = (0, exports.ValidateSecretRequest)(req, res);
    if (!result) {
        return;
    }
    const { id_token, secretID } = result;
    (0, exports.AuthenticateAndUpdateSecret)(id_token, secretID, req.body.data)
        .then(() => {
        res.sendStatus(200);
    })
        .catch((error) => {
        SchedulerOperations.FinishWithError(res, error);
    });
};
exports.ValidateAndUpdateSecret = ValidateAndUpdateSecret;
const CreateSecretIfNotExists = (orgID, userID, secretID) => {
    return new Promise((resolve, reject) => {
        const secretsManager = (0, exports.GetSecretsManager)();
        const parent = (0, exports.GetRootSecretParent)();
        const secretObtained = (0, exports.GetSecretID)(orgID, userID, secretID);
        return secretsManager.createSecret({
            parent,
            secretId: secretObtained,
            secret: {
                replication: {
                    automatic: {},
                }
            }
        })
            .then(() => {
            resolve();
        })
            .catch((error) => {
            exports.RunnerBackendLogger.emergContext({ orgID, userID }, `Error creating secret`, error);
            reject(error);
        });
    });
};
exports.CreateSecretIfNotExists = CreateSecretIfNotExists;
/**
 * function used to update secrets if you only have the firebase id_token instead of the orgID/userID
 * @param refresh_token goes into AuthenticateFirebaseTokenAndRetrieveTeamInfo to return the orgID/userID as 'fbTeamID' and 'fbUserID'
 * @param secretID is a piece of the whole secret ID that will be constructed later in the code flow
 * @param data can be anything
 * @returns
 */
const AuthenticateAndUpdateSecret = (refresh_token, secretID, data) => {
    return new Promise((resolve, reject) => {
        let orgID, userID;
        SchedulerOperations.AuthenticateFirebaseTokenAndRetrieveTeamInfoForScheduler(refresh_token, undefined)
            .then(data => {
            orgID = data.teamInfo.fbTeamID;
            userID = data.teamInfo.fbUserID;
        })
            .then(() => {
            return (0, exports.UpdateSecret)(orgID, userID, secretID, data);
        })
            .then(() => {
            resolve();
        }).catch((error) => {
            reject(error);
        });
    });
};
exports.AuthenticateAndUpdateSecret = AuthenticateAndUpdateSecret;
/**
 * Updates the data in a secret and returns an empty promise
 * @param orgID this and userID should be provided by a call to AuthenticateFirebaseTokenAndRetrieveTeamInfo higher in the code flow
 * @param userID
 * @param secretID is a piece of the whole secret ID that will be constructed later in the code flow
 * @param data can be anything
 * @returns
 */
const UpdateSecret = (orgID, userID, secretID, data) => {
    return new Promise((resolve, reject) => {
        (0, exports.CreateSecretIfNotExists)(orgID, userID, secretID)
            .then(() => {
            const parent = (0, exports.GetSecretParent)(orgID, userID, secretID);
            const secretsManager = (0, exports.GetSecretsManager)();
            // Payload is the plaintext data to store in the secret
            const payload = Buffer.from(JSON.stringify(data), "utf8");
            return secretsManager.addSecretVersion({
                parent: parent,
                payload: {
                    data: payload
                }
            });
        })
            .then(() => {
            resolve();
        })
            .catch((err) => {
            reject(err);
        });
    });
};
exports.UpdateSecret = UpdateSecret;
/**
 * ===== GET SECRETS =====
 */
/**
 * ValidateAndGet/Update/DeleteSecret are to be used if trying
 * to run these functions from an endpoint (unclear if these should
 * even be used)
 */
const ValidateAndGetSecret = (req, res) => {
    const result = (0, exports.ValidateSecretRequest)(req, res);
    if (!result) {
        return;
    }
    const { id_token, secretID } = result;
    (0, exports.AuthenticateFBAndGetSecret)(id_token, secretID)
        .then((result) => {
        res.json(result);
    })
        .catch((err) => {
        SchedulerOperations.FinishWithError(res, err);
    });
};
exports.ValidateAndGetSecret = ValidateAndGetSecret;
const ValidateSecretRequest = (req, res) => {
    if (!SchedulerOperations.isValidRequest(req, res, exports.SecretQueryParamsType, null))
        return null;
    const id_token = req.query.id_token;
    const secretID = req.query.secretID;
    if (!secretID) {
        res.send(new Error("has an unexpected secretID"));
        return null;
    }
    return { id_token, secretID };
};
exports.ValidateSecretRequest = ValidateSecretRequest;
const GetSecretID = (orgID, userID, secretID) => {
    return `${orgID}_${userID}_${!!secretID ? secretID : ""}`;
};
exports.GetSecretID = GetSecretID;
const GetRootSecretParent = () => {
    const parent = `projects/${Firebase.getFirebaseConfig().projectId}`;
    return parent;
};
exports.GetRootSecretParent = GetRootSecretParent;
const GetSecretParent = (orgID, userID, secretID) => {
    return `${(0, exports.GetRootSecretParent)()}/secrets/${(0, exports.GetSecretID)(orgID, userID, secretID)}`;
};
exports.GetSecretParent = GetSecretParent;
const GetSecretParentLatest = (orgID, userID, secretID) => {
    return `${(0, exports.GetSecretParent)(orgID, userID, secretID)}/versions/latest`;
};
exports.GetSecretParentLatest = GetSecretParentLatest;
/**
 * function used to get secrets if you only have the firebase id_token instead of the orgID and userID
 * @param refresh_token goes into AuthenticateFirebaseTokenAndRetrieveTeamInfo to return the orgID/userID as 'fbTeamID' and 'fbUserID'
 * @param secretID is a piece of the whole secret ID that will be constructed later in the code flow
 * @returns the stringified data from a secret
 */
const AuthenticateFBAndGetSecret = (refresh_token, secretID) => {
    return new Promise((resolve, reject) => {
        let orgID, userID;
        SchedulerOperations.AuthenticateFirebaseTokenAndRetrieveTeamInfoForScheduler(refresh_token, undefined)
            .then(data => {
            orgID = data.teamInfo.fbTeamID;
            userID = data.teamInfo.fbUserID;
        })
            .then(() => {
            return (0, exports.GetSecret)(orgID, userID, secretID);
        })
            .then((result) => {
            const payload = result[0].payload.data.toString();
            resolve(payload);
        })
            .catch((err) => {
            reject(err);
        });
    });
};
exports.AuthenticateFBAndGetSecret = AuthenticateFBAndGetSecret;
/**
 * Gets the data of a given secret in a promise
 * @param orgID this and userID should be provided by a call to AuthenticateFirebaseTokenAndRetrieveTeamInfo higher in the code flow
 * @param userID
 * @param secretID is a piece of the whole secret ID that will be constructed later in the code flow
 * @returns the stringified data from a secret in a promise
 */
const GetSecret = (orgID, userID, secretID) => {
    return new Promise((resolve, reject) => {
        (0, exports.GetSecretsManager)().accessSecretVersion({ name: (0, exports.GetSecretParentLatest)(orgID, userID, secretID) })
            .then((result) => {
            const payload = result[0].payload.data.toString();
            resolve(JSON.parse(payload));
        })
            .catch((err) => {
            reject(err);
        });
    });
};
exports.GetSecret = GetSecret;
/**
 * ===== DELETE SECRETS =====
 */
const ValidateAndDeleteSecret = (req, res) => {
    const result = (0, exports.ValidateSecretRequest)(req, res);
    if (!result) {
        return;
    }
    const { id_token, secretID } = result;
    (0, exports.AuthenticateAndDeleteSecret)(id_token, secretID)
        .then(() => {
        res.sendStatus(200);
    })
        .catch((err) => {
        SchedulerOperations.FinishWithError(res, err);
    });
};
exports.ValidateAndDeleteSecret = ValidateAndDeleteSecret;
/**
 * function used to delete secrets if you only have the firebase id_token instead of the orgID/userID
 * @param id_token goes into AuthenticateFirebaseTokenAndRetrieveTeamInfo to return the orgID/userID as 'fbTeamID' and 'fbUserID'
 * @param secretID is a piece of the whole secret ID that will be constructed later in the code flow
 * @returns
 */
const AuthenticateAndDeleteSecret = (refresh_token, secretID) => {
    return new Promise((resolve, reject) => {
        let orgID, userID;
        return SchedulerOperations.AuthenticateFirebaseTokenAndRetrieveTeamInfoForScheduler(refresh_token, undefined)
            .then(data => {
            orgID = data.teamInfo.fbTeamID;
            userID = data.teamInfo.fbUserID;
            return;
        })
            .then(() => {
            return (0, exports.DeleteSecret)(orgID, userID, secretID);
        })
            .then(() => {
            resolve();
        }).catch((error) => {
            reject(error);
        });
    });
};
exports.AuthenticateAndDeleteSecret = AuthenticateAndDeleteSecret;
/**
 * deletes a secret at a given secret ID in the google secrets manager, then resolves
 * @param orgID this and userID should be provided by a call to AuthenticateFirebaseTokenAndRetrieveTeamInfo higher in the code flow
 * @param userID
 * @param secretID is a piece of the whole secret ID that will be constructed later in the code flow
 * @returns a void promise
 */
const DeleteSecret = (orgID, userID, secretID) => {
    return new Promise((resolve, reject) => {
        return (0, exports.GetSecretsManager)().deleteSecret({ name: (0, exports.GetSecretParent)(orgID, userID, secretID) })
            .then(() => {
            resolve();
        })
            .catch((err) => {
            reject(err);
        });
    });
};
exports.DeleteSecret = DeleteSecret;
/**
 * Responsible for deleting secret if needed, then passing off the user connection deletion to the other helper
 * @param tokenData
 * @param environmentID
 * @returns
 */
const DeleteUserConnectionByEnvironmentAndConnectionType = (tokenData, environmentID, deleteAllFields) => {
    const { fbTeamID, fbUserID } = tokenData.teamInfo;
    let connectionTypeToUse;
    return new Promise((resolve, reject) => {
        // get the doc and see what connection type it is
        return ConnectionOperations.GetUserConnectionInfoByEnvironment(tokenData.firebase.firestore(), tokenData.teamInfo, environmentID)
            .then((connectionData) => {
            connectionTypeToUse = connectionData.connectionType;
            if (deleteAllFields) {
                return (0, exports.DeleteUserConnectionByEnvironment)(tokenData, environmentID);
            }
            else {
                return (0, exports.DeleteUserConnectionUsernameByEnvironment)(tokenData, environmentID, connectionTypeToUse);
            }
        })
            .then(() => {
            let connectionPromise;
            if (connectionTypeToUse === ConnectionOperations.EUserConnectionTypes.oauth) {
                connectionPromise = ConnectionOperations.GetCurrentUserSecretFromFirebase(tokenData, environmentID)
                    .then((connectionUID) => {
                    return ConnectionOperations.DeleteUserConnectionSecretUID(tokenData, environmentID)
                        .then(() => (0, exports.DeleteSecret)(tokenData.teamInfo.fbTeamID, tokenData.teamInfo.fbUserID, connectionUID));
                });
            }
            else {
                connectionPromise = Promise.resolve();
            }
            return connectionPromise;
        })
            .then(() => {
            resolve();
        })
            .catch(error => {
            exports.RunnerBackendLogger.errorContext({ orgID: fbTeamID, userID: fbUserID, envID: environmentID }, "DeleteUserConnectionByEnvironmentAndConnectionType", error);
            reject(error);
        });
    });
};
exports.DeleteUserConnectionByEnvironmentAndConnectionType = DeleteUserConnectionByEnvironmentAndConnectionType;
/**
 * Only responsible for deleting the users connection
 * @param tokenData
 * @param environmentID
 * @returns
 */
const DeleteUserConnectionByEnvironment = (tokenData, environmentID) => {
    const { teamInfo, firebase } = tokenData;
    const { fbTeamID, fbUserID } = teamInfo;
    return new Promise((resolve, reject) => {
        const resetConnection = ConnectionOperations.CreateBasicUserConnection(fbUserID);
        return ConnectionOperations.SetUserConnectionDataByEnvironment(firebase.firestore(), teamInfo, environmentID, resetConnection)
            .then(() => {
            resolve();
        })
            .catch((err) => {
            exports.RunnerBackendLogger.errorContext({ orgID: fbTeamID, userID: fbUserID, envID: environmentID }, "DeleteUserConnectionByEnvironment", err);
            reject();
        });
    });
};
exports.DeleteUserConnectionByEnvironment = DeleteUserConnectionByEnvironment;
const DeleteUserConnectionUsernameByEnvironment = (tokenData, environmentID, connectionType) => {
    const { teamInfo, firebase } = tokenData;
    const { fbTeamID, fbUserID } = teamInfo;
    return new Promise((resolve, reject) => {
        return ConnectionOperations.SetUserConnectionDataByEnvironment(firebase.firestore(), teamInfo, environmentID, { connectionType, connectionDetails: { user: "" } })
            .then(() => {
            resolve();
        })
            .catch((err) => {
            exports.RunnerBackendLogger.errorContext({ orgID: fbTeamID, userID: fbUserID, envID: environmentID }, "DeleteUserConnectionByEnvironment", err);
            reject(err);
        });
    });
};
exports.DeleteUserConnectionUsernameByEnvironment = DeleteUserConnectionUsernameByEnvironment;
//# sourceMappingURL=Secrets.js.map