"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useAddLocation = exports.listenMappingChangeCb = exports.getMappingCollectionRefAdmin = exports.getMappingCollectionRef = exports.LocationLogger = void 0;
const react_1 = require("react");
const CommonOperations = __importStar(require("./CommonOperations"));
const State = __importStar(require("./State"));
const TeamOperations = __importStar(require("./TeamOperations"));
const Firebase = __importStar(require("./Firebase"));
const Logging = __importStar(require("./Logging"));
exports.LocationLogger = Logging.GetLogger(Logging.LoggingArea.StorageLocations);
//Environments
const getMappingCollectionRef = (teamID) => {
    return CommonOperations.getOrgConfigDocRef(teamID).collection("mappings");
};
exports.getMappingCollectionRef = getMappingCollectionRef;
const getMappingCollectionRefAdmin = (teamID, firestore) => {
    return CommonOperations.getOrgConfigDocRefAdmin(firestore, teamID).collection("mappings");
};
exports.getMappingCollectionRefAdmin = getMappingCollectionRefAdmin;
const listenMappingChangeCb = (change, dispatch) => {
    const { type, doc } = change;
    if (type === "clear") {
        dispatch({ type: State.getClearAllEntityActionString(CommonOperations.DocRefs.mappings) });
    }
    else if (type === "added") {
        dispatch(State.getCreateMapping(parseInt(doc.id), doc.data()));
    }
    else if (type === "modified") { //need state/action
        dispatch(State.getModifyMapping(parseInt(doc.id), doc.data()));
    }
    else if (type === "removed") {
        dispatch(State.getDeleteMapping(parseInt(doc.id)));
    }
    else {
        console.log("unknown update type", type);
        console.assert(false);
    }
};
exports.listenMappingChangeCb = listenMappingChangeCb;
const useAddLocation = (teamID) => {
    const userID = TeamOperations.useUserID();
    const getNextLocationCounter = (0, react_1.useCallback)(() => {
        const firestore = Firebase.getFirestore();
        return CommonOperations.getNextCounter(firestore, CommonOperations.getOrgConfigDocRef(teamID), "locationCounter", 1);
    }, [teamID]);
    const addLocation = (0, react_1.useCallback)((storageMap) => {
        let newLocationCounter = 0;
        return getNextLocationCounter().then(nextStorageCounter => {
            newLocationCounter = nextStorageCounter;
            const obj = Object.assign({}, Object.assign(Object.assign({}, storageMap), { id: nextStorageCounter }));
            return (0, exports.getMappingCollectionRef)(teamID).doc(nextStorageCounter.toString()).set(obj);
        }).then(() => {
            exports.LocationLogger.infoContext({ orgID: teamID, userID }, `Added new location ${storageMap.locationName}`);
            return newLocationCounter;
        }).catch(error => {
            const newError = {
                errorString: error.message
            };
            throw newError;
        });
    }, [userID, teamID, getNextLocationCounter]);
    return addLocation;
};
exports.useAddLocation = useAddLocation;
//# sourceMappingURL=StorageMappings.js.map